@@ -11,4 +11,4 @@ int main(int argc, char ** argv)
 
 	return 0;
 }
-#endif
+#endif	/* WIN32 */
