@@ -17,6 +17,9 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#ifndef NUT_UPSSTATS_H_SEEN
+#define NUT_UPSSTATS_H_SEEN 1
+
 #ifdef __cplusplus
 /* *INDENT-OFF* */
 extern "C" {
@@ -35,3 +38,4 @@ typedef struct {
 /* *INDENT-ON* */
 #endif
 
+#endif	/* NUT_UPSSTATS_H_SEEN */
