@@ -29,7 +29,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Belkin Smart protocol driver"
-#define DRIVER_VERSION	"0.26"
+#define DRIVER_VERSION	"0.28"
 
 static ssize_t init_communication(void);
 static ssize_t get_belkin_reply(char *buf);
@@ -352,25 +352,12 @@ void upsdrv_updateinfo(void)
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
-	ssize_t	res;
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
 
-	res = init_communication();
-	if (res < 0) {
-		printf("Detection failed.  Trying a shutdown command anyway.\n");
-	}
-
-	/* tested on a F6C525-SER: this works when OL and OB */
-
-	/* shutdown type 2 (UPS system) */
-	send_belkin_command(CONTROL, "SDT", "2");
-
-	/* SDR means "do SDT and SDA, then reboot after n minutes" */
-	send_belkin_command(CONTROL, "SDR", "1");
-
-	printf("UPS should power off load in 5 seconds\n");
-
-	/* shutdown in 5 seconds */
-	send_belkin_command(CONTROL, "SDA", "5");
+	int	ret = do_loop_shutdown_commands("shutdown.return", NULL);
+	if (handling_upsdrv_shutdown > 0)
+		set_exit_flag(ret == STAT_INSTCMD_HANDLED ? EF_EXIT_SUCCESS : EF_EXIT_FAILURE);
 }
 
 /* handle "beeper.disable" */
@@ -432,6 +419,30 @@ static int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
+	if (!strcasecmp(cmdname, "shutdown.return")) {
+		ssize_t	res;
+
+		res = init_communication();
+		if (res < 0) {
+			printf("Detection failed.  Trying a shutdown command anyway.\n");
+		}
+
+		/* tested on a F6C525-SER: this works when OL and OB */
+
+		/* shutdown type 2 (UPS system) */
+		send_belkin_command(CONTROL, "SDT", "2");
+
+		/* SDR means "do SDT and SDA, then reboot after n minutes" */
+		send_belkin_command(CONTROL, "SDR", "1");
+
+		printf("UPS should power off load in 5 seconds\n");
+
+		/* shutdown in 5 seconds */
+		send_belkin_command(CONTROL, "SDA", "5");
+
+		return STAT_INSTCMD_HANDLED;
+	}
+
 	if (!strcasecmp(cmdname, "load.off")) {
 		do_off();
 		return STAT_INSTCMD_HANDLED;
@@ -530,6 +541,7 @@ void upsdrv_initinfo(void)
 
 	dstate_addcmd("beeper.disable");
 	dstate_addcmd("beeper.enable");
+	dstate_addcmd("shutdown.return");
 	dstate_addcmd("load.off");
 	dstate_addcmd("load.on");
 	dstate_addcmd("test.battery.start.quick");
