@@ -45,7 +45,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"Best Ferrups/Fortress driver"
-#define DRIVER_VERSION	"0.15"
+#define DRIVER_VERSION	"0.16"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -103,6 +103,7 @@ static struct {
 static int inverter_status;
 
 /* Forward decls */
+static int instcmd(const char *cmdname, const char *extra);
 
 /* Set up all the funky shared memory stuff used to communicate with upsd */
 void  upsdrv_initinfo (void)
@@ -158,6 +159,12 @@ void  upsdrv_initinfo (void)
 		fc.fullvolts,
 		fc.lowvolts,
 		fc.emptyvolts);
+
+	/* commands ----------------------------------------------- */
+	dstate_addcmd("shutdown.return");
+
+	/* install handlers */
+	upsh.instcmd = instcmd;
 }
 
 
@@ -461,12 +468,35 @@ static void ups_sync(void)
 	ser_get_line(upsfd, buf, sizeof(buf), '>', "\012", 3, 0);
 }
 
+/* handler for commands to be sent to UPS */
+static
+int instcmd(const char *cmdname, const char *extra)
+{
+	NUT_UNUSED_VARIABLE(extra);
+
+	if (!strcasecmp(cmdname, "shutdown.return")) {
+		/* NB: hard-wired password */
+		ser_send(upsfd, "pw377\r");
+		/* power off in 10 seconds and restart when line power returns,
+		 * FE7K required a min of 5 seconds for off to function */
+		ser_send(upsfd, "o 10 a\r");
+
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	return STAT_INSTCMD_UNKNOWN;
+}
+
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
-	/* NB: hard-wired password */
-	ser_send(upsfd, "pw377\r");
-	ser_send(upsfd, "o 10 a\r");	/* power off in 10 seconds and restart when line power returns, FE7K required a min of 5 seconds for off to function */
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
+	int	ret = do_loop_shutdown_commands("shutdown.return", NULL);
+	if (handling_upsdrv_shutdown > 0)
+		set_exit_flag(ret == STAT_INSTCMD_HANDLED ? EF_EXIT_SUCCESS : EF_EXIT_FAILURE);
 }
 
 /* list flags and values that you want to receive via -x */
