@@ -3,7 +3,8 @@
    Copyright (C)
 	2003		Russell Kroll <rkroll@exploits.org>
 	2008		Arjen de Korte <adkorte-guest@alioth.debian.org>
-	2012 - 2017	Arnaud Quette <arnaud.quette@free.fr>
+	2012-2017	Arnaud Quette <arnaud.quette@free.fr>
+	2020-2025	Jim Klimov <jimklimov+nut@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,16 +25,16 @@
 
 #include <stdio.h>
 #ifndef WIN32
-#include <stdarg.h>
-#include <sys/stat.h>
-#include <pwd.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#else
-#include <strings.h>
-#include "wincompat.h"
-#endif
+# include <stdarg.h>
+# include <sys/stat.h>
+# include <pwd.h>
+# include <sys/types.h>
+# include <sys/socket.h>
+# include <sys/un.h>
+#else	/* WIN32 */
+# include <strings.h>
+# include "wincompat.h"
+#endif	/* WIN32 */
 
 #include "common.h"
 #include "dstate.h"
@@ -45,15 +46,16 @@
 	static TYPE_FD	sockfd = ERROR_FD;
 #ifndef WIN32
 	static char	*sockfn = NULL;
-#else
-	static OVERLAPPED connect_overlapped;
+#else	/* WIN32 */
+	static OVERLAPPED	connect_overlapped;
 	static char	*pipename = NULL;
-#endif
-	static int	stale = 1, alarm_active = 0, ignorelb = 0;
-	static char	status_buf[ST_MAX_VALUE_LEN], alarm_buf[ST_MAX_VALUE_LEN];
-	static st_tree_t	*dtree_root = NULL;
+#endif	/* WIN32 */
+	static int	stale = 1, alarm_active = 0, alarm_status = 0, ignorelb = 0;
+	static char	status_buf[ST_MAX_VALUE_LEN], alarm_buf[ST_MAX_VALUE_LEN],
+			buzzmode_buf[ST_MAX_VALUE_LEN];
 	static conn_t	*connhead = NULL;
-	static cmdlist_t *cmdhead = NULL;
+	static st_tree_t	*dtree_root = NULL;
+	static cmdlist_t	*cmdhead = NULL;
 
 	struct ups_handler	upsh;
 
@@ -105,6 +107,9 @@ static void sock_fail(const char *fn)
 		printf(" - rm %s\n\n", dflt_statepath());
 		printf(" - mkdir %s\n", dflt_statepath());
 		break;
+
+	default:
+		break;
 	}
 
 	/*
@@ -115,7 +120,7 @@ static void sock_fail(const char *fn)
 	printf("\n");
 	fatalx(EXIT_FAILURE, "Exiting.");
 }
-#endif
+#endif	/* !WIN32 */
 
 static TYPE_FD sock_open(const char *fn)
 {
@@ -162,6 +167,9 @@ static TYPE_FD sock_open(const char *fn)
 		fatal_with_errno(EXIT_FAILURE, "listen(%d, %d) failed", fd, DS_LISTEN_BACKLOG);
 	}
 
+	if (!getenv("NUT_QUIET_INIT_LISTENER"))
+		upslogx(LOG_INFO, "Listening on socket %s", sockfn);
+
 #else /* WIN32 */
 
 	fd = CreateNamedPipe(
@@ -183,18 +191,22 @@ static TYPE_FD sock_open(const char *fn)
 	}
 
 	/* Prepare an async wait on a connection on the pipe */
-	memset(&connect_overlapped,0,sizeof(connect_overlapped));
+	memset(&connect_overlapped, 0, sizeof(connect_overlapped));
 	connect_overlapped.hEvent = CreateEvent(NULL, /*Security*/
 			FALSE, /* auto-reset*/
 			FALSE, /* inital state = non signaled*/
 			NULL /* no name*/);
-	if(connect_overlapped.hEvent == NULL ) {
+	if (connect_overlapped.hEvent == NULL) {
 		fatal_with_errno(EXIT_FAILURE, "Can't create event");
 	}
 
 	/* Wait for a connection */
-	ConnectNamedPipe(fd,&connect_overlapped);
-#endif
+	ConnectNamedPipe(fd, &connect_overlapped);
+
+	if (!getenv("NUT_QUIET_INIT_LISTENER"))
+		upslogx(LOG_INFO, "Listening on named pipe %s", fn);
+
+#endif	/* WIN32 */
 
 	return fd;
 }
@@ -202,18 +214,22 @@ static TYPE_FD sock_open(const char *fn)
 static void sock_disconnect(conn_t *conn)
 {
 #ifndef WIN32
+	upsdebugx(3, "%s: disconnecting socket %d", __func__, (int)conn->fd);
 	close(conn->fd);
-#else
-	/* FIXME not sure if this is the right way to close a connection */
-	if( conn->read_overlapped.hEvent != INVALID_HANDLE_VALUE) {
+#else	/* WIN32 */
+	/* FIXME NUT_WIN32_INCOMPLETE not sure if this is the right way to close a connection */
+	if (conn->read_overlapped.hEvent != INVALID_HANDLE_VALUE) {
 		CloseHandle(conn->read_overlapped.hEvent);
 		conn->read_overlapped.hEvent = INVALID_HANDLE_VALUE;
 	}
+	upsdebugx(3, "%s: disconnecting named pipe handle %p", __func__, conn->fd);
 	DisconnectNamedPipe(conn->fd);
-#endif
+#endif	/* WIN32 */
 
+	upsdebugx(5, "%s: finishing parsing context", __func__);
 	pconf_finish(&conn->ctx);
 
+	upsdebugx(5, "%s: relinking the chain of connections", __func__);
 	if (conn->prev) {
 		conn->prev->next = conn->next;
 	} else {
@@ -226,6 +242,7 @@ static void sock_disconnect(conn_t *conn)
 		/* conntail = conn->prev; */
 	}
 
+	upsdebugx(5, "%s: freeing the conn object", __func__);
 	free(conn);
 }
 
@@ -275,32 +292,32 @@ static void send_to_all(const char *fmt, ...)
 
 #ifndef WIN32
 		ret = write(conn->fd, buf, buflen);
-#else
+#else	/* WIN32 */
 		DWORD bytesWritten = 0;
 		BOOL  result = FALSE;
 
 		result = WriteFile (conn->fd, buf, buflen, &bytesWritten, NULL);
 		if( result == 0 ) {
-			upsdebugx(2, "write failed on handle %p, disconnecting", conn->fd);
+			upsdebugx(2, "%s: write failed on handle %p, disconnecting", __func__, conn->fd);
 			sock_disconnect(conn);
 			continue;
 		}
 		else  {
 			ret = (ssize_t)bytesWritten;
 		}
-#endif
+#endif	/* WIN32 */
 
 		if ((ret < 1) || (ret != (ssize_t)buflen)) {
 #ifndef WIN32
-			upsdebugx(0, "WARNING: %s: write %" PRIiSIZE " bytes to "
-				"socket %d failed (ret=%" PRIiSIZE "), disconnecting: %s",
-				__func__, buflen, (int)conn->fd, ret, strerror(errno));
-#else
-			upsdebugx(0, "WARNING: %s: write %" PRIiSIZE " bytes to "
-				"handle %p failed (ret=%" PRIiSIZE "), disconnecting: %s",
-				__func__, buflen, conn->fd, ret, strerror(errno));
-#endif
-			upsdebugx(6, "failed write: %s", buf);
+			upsdebug_with_errno(0, "WARNING: %s: write %" PRIuSIZE " bytes to "
+				"socket %d failed (ret=%" PRIiSIZE "), disconnecting.",
+				__func__, buflen, (int)conn->fd, ret);
+#else	/* WIN32 */
+			upsdebug_with_errno(0, "WARNING: %s: write %" PRIuSIZE " bytes to "
+				"handle %p failed (ret=%" PRIiSIZE "), disconnecting.",
+				__func__, buflen, conn->fd, ret);
+#endif	/* WIN32 */
+			upsdebugx(6, "%s: failed write: %s", __func__, buf);
 
 			sock_disconnect(conn);
 
@@ -315,7 +332,7 @@ static void send_to_all(const char *fmt, ...)
 			dstate_setinfo("driver.parameter.synchronous", "%s",
 				(do_synchronous==1)?"yes":((do_synchronous==0)?"no":"auto"));
 		} else {
-			upsdebugx(6, "%s: write %" PRIiSIZE " bytes to socket %d succeeded "
+			upsdebugx(6, "%s: write %" PRIuSIZE " bytes to socket %d succeeded "
 				"(ret=%" PRIiSIZE "): %s",
 				__func__, buflen, conn->fd, ret, buf);
 		}
@@ -331,7 +348,7 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 #ifdef WIN32
 	DWORD bytesWritten = 0;
 	BOOL  result = FALSE;
-#endif
+#endif	/* WIN32 */
 
 	va_start(ap, fmt);
 #ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
@@ -372,7 +389,7 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 
 #ifndef WIN32
 	ret = write(conn->fd, buf, buflen);
-#else
+#else	/* WIN32 */
 	result = WriteFile (conn->fd, buf, buflen, &bytesWritten, NULL);
 	if( result == 0 ) {
 		ret = 0;
@@ -380,29 +397,25 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 	else  {
 		ret = (ssize_t)bytesWritten;
 	}
-#endif
+#endif	/* WIN32 */
 
 	if (ret < 0) {
 		/* Hacky bugfix: throttle down for upsd to read that */
 #ifndef WIN32
-		upsdebugx(1, "%s: had to throttle down to retry "
-			"writing %" PRIiSIZE " bytes to socket %d "
-			"(ret=%" PRIiSIZE ", errno=%d, strerror=%s): %s",
-			__func__, buflen, (int)conn->fd,
-			ret, errno, strerror(errno),
-			buf);
-#else
-		upsdebugx(1, "%s: had to throttle down to retry "
-			"writing %" PRIiSIZE " bytes to handle %p "
-			"(ret=%" PRIiSIZE ", errno=%d, strerror=%s): %s",
-			__func__, buflen, conn->fd,
-			ret, errno, strerror(errno),
-			buf);
-#endif
+		upsdebug_with_errno(1, "%s: had to throttle down to retry "
+			"writing %" PRIuSIZE " bytes to socket %d (ret=%" PRIiSIZE ") : %s",
+			__func__, buflen, (int)conn->fd, ret, buf);
+#else	/* WIN32 */
+		upsdebug_with_errno(1, "%s: had to throttle down to retry "
+			"writing %" PRIuSIZE " bytes to handle %p (ret=%" PRIiSIZE ") : %s",
+			__func__, buflen, conn->fd, ret, buf);
+#endif	/* WIN32 */
+
 		usleep(200);
+
 #ifndef WIN32
 		ret = write(conn->fd, buf, buflen);
-#else
+#else	/* WIN32 */
 		result = WriteFile (conn->fd, buf, buflen, &bytesWritten, NULL);
 		if( result == 0 ) {
 			ret = 0;
@@ -410,7 +423,7 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 		else  {
 			ret = (ssize_t)bytesWritten;
 		}
-#endif
+#endif	/* WIN32 */
 		if (ret == (ssize_t)buflen) {
 			upsdebugx(1, "%s: throttling down helped", __func__);
 		}
@@ -418,15 +431,15 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 
 	if ((ret < 1) || (ret != (ssize_t)buflen)) {
 #ifndef WIN32
-		upsdebugx(0, "WARNING: %s: write %" PRIiSIZE " bytes to "
-			"socket %d failed (ret=%" PRIiSIZE "), disconnecting: %s",
-			__func__, buflen, (int)conn->fd, ret, strerror(errno));
-#else
-		upsdebugx(0, "WARNING: %s: write %" PRIiSIZE " bytes to "
-			"handle %p failed (ret=%" PRIiSIZE "), disconnecting: %s",
-			__func__, buflen, conn->fd, ret, strerror(errno));
-#endif
-		upsdebugx(6, "failed write: %s", buf);
+		upsdebug_with_errno(0, "WARNING: %s: write %" PRIuSIZE " bytes to "
+			"socket %d failed (ret=%" PRIiSIZE "), disconnecting.",
+			__func__, buflen, (int)conn->fd, ret);
+#else	/* WIN32 */
+		upsdebug_with_errno(0, "WARNING: %s: write %" PRIuSIZE " bytes to "
+			"handle %p failed (ret=%" PRIiSIZE "), disconnecting.",
+			__func__, buflen, conn->fd, ret);
+#endif	/* WIN32 */
+		upsdebugx(6, "%s: failed write: %s", __func__, buf);
 		sock_disconnect(conn);
 
 		/* TOTHINK: Maybe fallback elsewhere in other cases? */
@@ -443,14 +456,14 @@ static int send_to_one(conn_t *conn, const char *fmt, ...)
 		return 0;	/* failed */
 	} else {
 #ifndef WIN32
-		upsdebugx(6, "%s: write %" PRIiSIZE " bytes to socket %d succeeded "
+		upsdebugx(6, "%s: write %" PRIuSIZE " bytes to socket %d succeeded "
 			"(ret=%" PRIiSIZE "): %s",
 			__func__, buflen, conn->fd, ret, buf);
-#else
-		upsdebugx(6, "%s: write %" PRIiSIZE " bytes to handle %p succeeded "
+#else	/* WIN32 */
+		upsdebugx(6, "%s: write %" PRIuSIZE " bytes to handle %p succeeded "
 			"(ret=%" PRIiSIZE "): %s",
 			__func__, buflen, conn->fd, ret, buf);
-#endif
+#endif	/* WIN32 */
 	}
 
 	return 1;	/* OK */
@@ -465,16 +478,16 @@ static void sock_connect(TYPE_FD sock)
 	int	fd;
 
 	struct sockaddr_un sa;
-#if defined(__hpux) && !defined(_XOPEN_SOURCE_EXTENDED)
+# if defined(__hpux) && !defined(_XOPEN_SOURCE_EXTENDED)
 	int	salen;
-#else
+# else
 	socklen_t	salen;
-#endif
+# endif
 	salen = sizeof(sa);
 	fd = accept(sock, (struct sockaddr *) &sa, &salen);
 
 	if (INVALID_FD(fd)) {
-		upslog_with_errno(LOG_ERR, "accept on unix fd failed");
+		upslog_with_errno(LOG_ERR, "%s: accept on unix fd failed", __func__);
 		return;
 	}
 
@@ -490,7 +503,7 @@ static void sock_connect(TYPE_FD sock)
 		ret = fcntl(fd, F_GETFL, 0);
 
 		if (ret < 0) {
-			upslog_with_errno(LOG_ERR, "fcntl get on unix fd failed");
+			upslog_with_errno(LOG_ERR, "%s: fcntl get on unix fd failed", __func__);
 			close(fd);
 			return;
 		}
@@ -498,7 +511,7 @@ static void sock_connect(TYPE_FD sock)
 		ret = fcntl(fd, F_SETFL, ret | O_NDELAY);
 
 		if (ret < 0) {
-			upslog_with_errno(LOG_ERR, "fcntl set O_NDELAY on unix fd failed");
+			upslog_with_errno(LOG_ERR, "%s: fcntl set O_NDELAY on unix fd failed", __func__);
 			close(fd);
 			return;
 		}
@@ -566,9 +579,11 @@ static void sock_connect(TYPE_FD sock)
 	ReadFile (conn->fd, conn->buf,
 		sizeof(conn->buf) - 1, /* -1 to be sure to have a trailling 0 */
 		NULL, &(conn->read_overlapped));
-#endif
+#endif	/* WIN32 */
 
 	conn->nobroadcast = 0;
+	conn->readzero = 0;
+	conn->closing = 0;
 	pconf_init(&conn->ctx, NULL);
 
 	if (connhead) {
@@ -579,31 +594,18 @@ static void sock_connect(TYPE_FD sock)
 	connhead = conn;
 
 #ifndef WIN32
-	upsdebugx(3, "new connection on fd %d", fd);
-#else
-	upsdebugx(3, "new connection on handle %p", sock);
-#endif
+	upsdebugx(3, "%s: new connection on fd %d", __func__, fd);
+#else	/* WIN32 */
+	upsdebugx(3, "%s: new connection on handle %p", __func__, sock);
+#endif	/* WIN32 */
 
 }
 
-static int st_tree_dump_conn(st_tree_t *node, conn_t *conn)
+static int st_tree_dump_conn_one_node(st_tree_t *node, conn_t *conn)
 {
-	int	ret;
 	enum_t	*etmp;
 	range_t	*rtmp;
 
-	if (!node) {
-		return 1;	/* not an error */
-	}
-
-	if (node->left) {
-		ret = st_tree_dump_conn(node->left, conn);
-
-		if (!ret) {
-			return 0;	/* write failed in the child */
-		}
-	}
-
 	if (!send_to_one(conn, "SETINFO %s \"%s\"\n", node->var, node->val)) {
 		return 0;	/* write failed, bail out */
 	}
@@ -651,6 +653,28 @@ static int st_tree_dump_conn(st_tree_t *node, conn_t *conn)
 		}
 	}
 
+	return 1;	/* everything's OK here ... */
+}
+
+static int st_tree_dump_conn(st_tree_t *node, conn_t *conn)
+{
+	int	ret;
+
+	if (!node) {
+		return 1;	/* not an error */
+	}
+
+	if (node->left) {
+		ret = st_tree_dump_conn(node->left, conn);
+
+		if (!ret) {
+			return 0;	/* write failed in the child */
+		}
+	}
+
+	if (!st_tree_dump_conn_one_node(node, conn))
+		return 0;	/* one of writes failed, bail out */
+
 	if (node->right) {
 		return st_tree_dump_conn(node->right, conn);
 	}
@@ -681,28 +705,62 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 {
 #ifdef WIN32
 	char *sockfn = pipename;	/* Just for the report below; not a global var in WIN32 builds */
-#endif
+#endif	/* WIN32 */
 
-	upsdebugx(6, "Driver on %s is now handling %s with %" PRIuSIZE " args",
-		sockfn, numarg ? arg[0] : "<skipped: no command>", numarg);
+	upsdebugx(6, "%s: Driver on %s is now handling %s with %" PRIuSIZE " args",
+		__func__, sockfn, numarg ? arg[0] : "<skipped: no command>", numarg);
 
 	if (numarg < 1) {
 		return 0;
 	}
 
-	if (!strcasecmp(arg[0], "DUMPALL")) {
+	if (!strcasecmp(arg[0], "LOGOUT")) {
+		send_to_one(conn, "OK Goodbye\n");
+#ifndef WIN32
+		upsdebugx(2, "%s: received LOGOUT on socket %d, will be disconnecting", __func__, (int)conn->fd);
+#else	/* WIN32 */
+		upsdebugx(2, "%s: received LOGOUT on handle %p, will be disconnecting", __func__, conn->fd);
+#endif	/* WIN32 */
+		/* Let the system flush the reply somehow (or the other
+		 * side to just see it) before we drop the pipe */
+		usleep(1000000);
+		/* err on the safe side, and actually close/free conn separately */
+		conn->closing = 1;
+		upsdebugx(4, "%s: LOGOUT processing finished", __func__);
+		return 2;
+	}
 
+	if (!strcasecmp(arg[0], "GETPID")) {
+		send_to_one(conn, "PID %" PRIiMAX "\n", (intmax_t)getpid());
+		return 1;
+	}
+
+	if (!strcasecmp(arg[0], "DUMPALL") || !strcasecmp(arg[0], "DUMPSTATUS") || (!strcasecmp(arg[0], "DUMPVALUE") && numarg > 1)) {
 		/* first thing: the staleness flag (see also below) */
 		if ((stale == 1) && !send_to_one(conn, "DATASTALE\n")) {
 			return 1;
 		}
 
-		if (!st_tree_dump_conn(dtree_root, conn)) {
-			return 1;
-		}
+		if (!strcasecmp(arg[0], "DUMPALL")) {
+			if (!st_tree_dump_conn(dtree_root, conn)) {
+				return 1;
+			}
 
-		if (!cmd_dump_conn(conn)) {
-			return 1;
+			if (!cmd_dump_conn(conn)) {
+				return 1;
+			}
+		} else {
+			/* A cheaper version of the dump */
+			char	*varname = (!strcasecmp(arg[0], "DUMPSTATUS") ? "ups.status" : (numarg > 1 ? arg[1] : NULL));
+			st_tree_t	*sttmp = (varname ? state_tree_find(dtree_root, varname) : NULL);
+
+			if (!sttmp) {
+				upsdebugx(1, "%s: %s was requested but currently no %s is known",
+					__func__, arg[0], NUT_STRARG(varname));
+			} else {
+				if (!st_tree_dump_conn_one_node(sttmp, conn))
+					return 1;
+			}
 		}
 
 		if ((stale == 0) && !send_to_one(conn, "DATAOK\n")) {
@@ -723,9 +781,9 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 		conn->nobroadcast = 1;
 #ifndef WIN32
 		snprintf(buf, sizeof(buf), "socket %d", conn->fd);
-#else
+#else	/* WIN32 */
 		snprintf(buf, sizeof(buf), "handle %p", conn->fd);
-#endif
+#endif	/* WIN32 */
 		upsdebugx(1, "%s: %s requested NOBROADCAST mode",
 			__func__, buf);
 		return 1;
@@ -742,9 +800,9 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 		}
 #ifndef WIN32
 		snprintf(buf, sizeof(buf), "socket %d", conn->fd);
-#else
+#else	/* WIN32 */
 		snprintf(buf, sizeof(buf), "handle %p", conn->fd);
-#endif
+#endif	/* WIN32 */
 		upsdebugx(1,
 			"%s: %s requested %sBROADCAST mode",
 			__func__, buf,
@@ -763,7 +821,7 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 		char *cmdparam = NULL;
 		char *cmdid = NULL;
 
-		/* Check if <cmdparam> and TRACKING were provided */
+		/* Check if <cmdparam> and/or TRACKING were provided */
 		if (numarg == 3) {
 			cmdparam = arg[2];
 		} else if (numarg == 4 && !strcasecmp(arg[2], "TRACKING")) {
@@ -807,7 +865,27 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 			return 1;
 		}
 
-		upslogx(LOG_NOTICE, "Got INSTCMD, but driver lacks a handler");
+		if (cmdparam) {
+			upslogx(LOG_NOTICE,
+				"Got INSTCMD '%s' '%s', but driver lacks a handler",
+				NUT_STRARG(cmdname), NUT_STRARG(cmdparam));
+		} else {
+			upslogx(LOG_NOTICE,
+				"Got INSTCMD '%s', but driver lacks a handler",
+				NUT_STRARG(cmdname));
+		}
+
+		/* Send back execution result (here, STAT_INSTCMD_UNKNOWN)
+		 * if TRACKING was requested.
+		 * Note that in practice we should not get here often: if the
+		 * instcmd was not registered, it may be rejected earlier in
+		 * call stack, or returned by a driver's handler (for unknown
+		 * commands) just a bit above.
+		 */
+		if (cmdid)
+			send_tracking(conn, cmdid, ret);
+
+		/* The command was handled, status is a separate consideration */
 		return 1;
 	}
 
@@ -872,6 +950,7 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 static void sock_read(conn_t *conn)
 {
 	ssize_t	ret, i;
+	int	ret_arg = -1;
 
 #ifndef WIN32
 	char	buf[SMALLBUF];
@@ -890,7 +969,41 @@ static void sock_read(conn_t *conn)
 			return;
 		}
 	}
-#else
+
+	if (ret == 0) {
+		int	flags = fcntl(conn->fd, F_GETFL), is_closed = 0;
+		upsdebugx(2, "%s: read() returned 0; flags=%04X O_NDELAY=%04X",
+			__func__, (unsigned int)flags, (unsigned int)O_NDELAY);
+		if (flags & O_NDELAY || O_NDELAY == 0) {
+			/* O_NDELAY with zero bytes means nothing to read but
+			 * since read() follows a successful select() with
+			 * ready file descriptor, ret shouldn't be 0.
+			 * This may also mean that the counterpart has exited
+			 * and the file descriptor should be reaped.
+			 * e.g. a `driver -c reload -a testups` fires its
+			 * message over Unix socket and disconnects.
+			 */
+			is_closed = 1;
+		} else {
+			/* assume we will soon have data waiting in the buffer */
+			conn->readzero++;
+			upsdebugx(1, "%s: got zero-sized reads %d times in a row", __func__, conn->readzero);
+			if (conn->readzero > DSTATE_CONN_READZERO_THROTTLE_MAX) {
+				is_closed = 2;
+			} else {
+				usleep(DSTATE_CONN_READZERO_THROTTLE_USEC);
+			}
+		}
+
+		if (is_closed) {
+			upsdebugx(1, "%s: it seems the other side has closed the connection", __func__);
+			sock_disconnect(conn);
+			return;
+		}
+	} else {
+		conn->readzero = 0;
+	}
+#else	/* WIN32 */
 	char *buf = conn->buf;
 	DWORD bytesRead;
 	BOOL res;
@@ -907,7 +1020,7 @@ static void sock_read(conn_t *conn)
 		set_exit_flag(1);
 		return;
 	}
-#endif
+#endif	/* WIN32 */
 
 	for (i = 0; i < ret; i++) {
 
@@ -917,7 +1030,8 @@ static void sock_read(conn_t *conn)
 			continue;
 
 		case 1: /* try to use it, and complain about unknown commands */
-			if (!sock_arg(conn, conn->ctx.numargs, conn->ctx.arglist)) {
+			ret_arg = sock_arg(conn, conn->ctx.numargs, conn->ctx.arglist);
+			if (!ret_arg) {
 				size_t	arg;
 
 				upslogx(LOG_INFO, "Unknown command on socket: ");
@@ -925,7 +1039,13 @@ static void sock_read(conn_t *conn)
 				for (arg = 0; arg < conn->ctx.numargs && arg < INT_MAX; arg++) {
 					upslogx(LOG_INFO, "arg %d: %s", (int)arg, conn->ctx.arglist[arg]);
 				}
+			} else if (ret_arg == 2) {
+				/* closed by LOGOUT processing, conn is free()'d */
+				if (i < ret)
+					upsdebugx(1, "%s: returning early, socket may be not valid anymore", __func__);
+				return;
 			}
+
 			continue;
 
 		default: /* nothing parsed */
@@ -938,7 +1058,7 @@ static void sock_read(conn_t *conn)
 	/* Restart async read */
 	memset(conn->buf,0,sizeof(conn->buf));
 	ReadFile(conn->fd,conn->buf,sizeof(conn->buf)-1,NULL,&(conn->read_overlapped)); /* -1 to be sure to have a trailling 0 */
-#endif
+#endif	/* WIN32 */
 }
 
 static void sock_close(void)
@@ -954,10 +1074,10 @@ static void sock_close(void)
 			free(sockfn);
 			sockfn = NULL;
 		}
-#else
+#else	/* WIN32 */
 		FlushFileBuffers(sockfd);
 		CloseHandle(sockfd);
-#endif
+#endif	/* WIN32 */
 
 		sockfd = ERROR_FD;
 	}
@@ -975,7 +1095,7 @@ static void sock_close(void)
 
 char * dstate_init(const char *prog, const char *devname)
 {
-	char	sockname[SMALLBUF];
+	char	sockname[NUT_PATH_MAX + 1];
 
 #ifndef WIN32
 	/* do this here for now */
@@ -993,19 +1113,19 @@ char * dstate_init(const char *prog, const char *devname)
 	} else {
 		snprintf(sockname, sizeof(sockname), "%s/%s", dflt_statepath(), prog);
 	}
-#else
+#else	/* WIN32 */
 	/* upsname (and so devname) is now mandatory so no need to test it */
 	snprintf(sockname, sizeof(sockname), "\\\\.\\pipe\\%s-%s", prog, devname);
 	pipename = xstrdup(sockname);
-#endif
+#endif	/* WIN32 */
 
 	sockfd = sock_open(sockname);
 
 #ifndef WIN32
-	upsdebugx(2, "dstate_init: sock %s open on fd %d", sockname, sockfd);
-#else
-	upsdebugx(2, "dstate_init: sock %s open on handle %p", sockname, sockfd);
-#endif
+	upsdebugx(2, "%s: sock %s open on fd %d", __func__, sockname, sockfd);
+#else	/* WIN32 */
+	upsdebugx(2, "%s: sock %s open on handle %p", __func__, sockname, sockfd);
+#endif	/* WIN32 */
 
 	/* NOTE: Caller must free this string */
 	return xstrdup(sockname);
@@ -1016,13 +1136,12 @@ int dstate_poll_fds(struct timeval timeout, TYPE_FD arg_extrafd)
 {
 	int	maxfd = 0; /* Unidiomatic use vs. "sockfd" below, which is "int" on non-WIN32 */
 	int	overrun = 0;
-	conn_t	*conn;
+	conn_t	*conn, *cnext;
 	struct timeval	now;
 
 #ifndef WIN32
 	int	ret;
 	fd_set	rfds;
-	conn_t	*cnext;
 
 	FD_ZERO(&rfds);
 	FD_SET(sockfd, &rfds);
@@ -1077,7 +1196,7 @@ int dstate_poll_fds(struct timeval timeout, TYPE_FD arg_extrafd)
 			break;
 
 		default:
-			upslog_with_errno(LOG_ERR, "select unix sockets failed");
+			upslog_with_errno(LOG_ERR, "%s: select unix sockets failed", __func__);
 		}
 
 		return overrun;
@@ -1095,6 +1214,14 @@ int dstate_poll_fds(struct timeval timeout, TYPE_FD arg_extrafd)
 		}
 	}
 
+	for (conn = connhead; conn; conn = cnext) {
+		cnext = conn->next;
+
+		if (conn->closing) {
+			sock_disconnect(conn);
+		}
+	}
+
 	/* tell the caller if that fd woke up */
 	if (VALID_FD(arg_extrafd) && (FD_ISSET(arg_extrafd, &rfds))) {
 		return 1;
@@ -1154,7 +1281,7 @@ int dstate_poll_fds(struct timeval timeout, TYPE_FD arg_extrafd)
 	}
 
 	if (ret == WAIT_FAILED) {
-		upslog_with_errno(LOG_ERR, "waitfor failed");
+		upslog_with_errno(LOG_ERR, "%s: waitfor failed", __func__);
 		return overrun;
 	}
 
@@ -1176,13 +1303,21 @@ int dstate_poll_fds(struct timeval timeout, TYPE_FD arg_extrafd)
 		}
 	}
 
+	for (conn = connhead; conn; conn = cnext) {
+		cnext = conn->next;
+
+		if (conn->closing) {
+			sock_disconnect(conn);
+		}
+	}
+
 	/* tell the caller if that fd woke up */
 /*
 	if (VALID_FD(arg_extrafd) && (ret == arg_extrafd)) {
 		return 1;
 	}
 */
-#endif
+#endif	/* WIN32 */
 
 	return overrun;
 }
@@ -1355,7 +1490,7 @@ void dstate_setaux(const char *var, long aux)
 	sttmp = state_tree_find(dtree_root, var);
 
 	if (!sttmp) {
-		upslogx(LOG_ERR, "dstate_setaux: base variable (%s) does not exist", var);
+		upslogx(LOG_ERR, "%s: base variable (%s) does not exist", __func__, var);
 		return;
 	}
 
@@ -1508,22 +1643,53 @@ void status_init(void)
 	}
 
 	memset(status_buf, 0, sizeof(status_buf));
+	alarm_status = 0;
+}
+
+/* check if a status element has been set, return 0 if not, 1 if yes
+ * (considering a whole-word token in temporary status_buf) */
+int status_get(const char *buf)
+{
+	return str_contains_token(status_buf, buf);
 }
 
 /* add a status element */
-void status_set(const char *buf)
+static int status_set_callback(char *tgt, size_t tgtsize, const char *token)
 {
-	if (ignorelb && !strcasecmp(buf, "LB")) {
-		upsdebugx(2, "%s: ignoring LB flag from device", __func__);
-		return;
+	if (tgt != status_buf || tgtsize != sizeof(status_buf)) {
+		upsdebugx(2, "%s: called for wrong use-case", __func__);
+		return 0;
 	}
 
-	/* separate with a space if multiple elements are present */
-	if (strlen(status_buf) > 0) {
-		snprintfcat(status_buf, sizeof(status_buf), " %s", buf);
-	} else {
-		snprintfcat(status_buf, sizeof(status_buf), "%s", buf);
+	if (ignorelb && !strcasecmp(token, "LB")) {
+		upsdebugx(2, "%s: ignoring LB flag from device", __func__);
+		return 0;
 	}
+
+	if (!strcasecmp(token, "ALARM")) {
+		/* Drivers really should not raise alarms this way,
+		 * but for the sake of third-party forks, we handle
+		 * the possibility...
+		 */
+		upsdebugx(2, "%s: (almost) ignoring ALARM set as a status", __func__);
+		if (!alarm_status && !alarm_active && strlen(alarm_buf) == 0) {
+			alarm_init();	/* no-op currently, but better be proper about it */
+			alarm_set("[N/A]");
+		}
+		alarm_status++;
+		return 0;
+	}
+
+	/* Proceed adding the token */
+	return 1;
+}
+
+void status_set(const char *buf)
+{
+#ifdef DEBUG
+	upsdebugx(3, "%s: '%s'\n", __func__, buf);
+#endif
+	str_add_unique_token(status_buf, sizeof(status_buf), buf, status_set_callback, NULL);
 }
 
 /* write the status_buf into the externally visible dstate storage */
@@ -1554,6 +1720,32 @@ void status_commit(void)
 		break;
 	}
 
+	/* NOTE: Not sure if any clients rely on ALARM being first if raised,
+	 * but note that if someone also uses status_set("ALARM") we can end
+	 * up with a "[N/A]" alarm value injected (if no other alarm was set)
+	 * and only add the token here so it remains first.
+	 *
+	 * NOTE: alarm_commit() must be executed before status_commit() for
+	 * this report to work!
+	 * * If a driver only called status_set("ALARM") and did not bother
+	 *   with alarm_commit(), the "ups.alarm" value queries would have
+	 *   returned NULL if not for the "sloppy driver" fix below, although
+	 *   the "ups.status" value would report an ALARM token.
+	 * * If a driver properly used alarm_init() and alarm_set(), but then
+	 *   called status_commit() before alarm_commit(), the "ups.status"
+	 *   value would not know to report an ALARM token, as before.
+	 * * If a driver used both status_set("ALARM") and alarm_set() later,
+	 *   the injected "[N/A]" value of the alarm (if that's its complete
+	 *   value) would be overwritten by the explicitly assigned contents,
+	 *   and an explicit alarm_commit() would be required for proper
+	 *   reporting from a non-sloppy driver.
+	 */
+
+	if (!alarm_active && alarm_status && !strcmp(alarm_buf, "[N/A]")) {
+		upsdebugx(2, "%s: Assume sloppy driver coding that ignored alarm methods and used status_set(\"ALARM\") instead: commit the injected N/A ups.alarm value", __func__);
+		alarm_commit();
+	}
+
 	if (alarm_active) {
 		dstate_setinfo("ups.status", "ALARM %s", status_buf);
 	} else {
@@ -1561,6 +1753,33 @@ void status_commit(void)
 	}
 }
 
+/* similar functions for experimental.ups.mode.buzzwords, where tracked
+ * dynamically (e.g. due to ECO/ESS/HE/Smart modes supported by the device) */
+void buzzmode_init(void)
+{
+	memset(buzzmode_buf, 0, sizeof(buzzmode_buf));
+}
+
+int  buzzmode_get(const char *buf)
+{
+	return str_contains_token(buzzmode_buf, buf);
+}
+
+void buzzmode_set(const char *buf)
+{
+	str_add_unique_token(buzzmode_buf, sizeof(buzzmode_buf), buf, NULL, NULL);
+}
+
+void buzzmode_commit(void)
+{
+	if (!*buzzmode_buf) {
+		dstate_delinfo("experimental.ups.mode.buzzwords");
+		return;
+	}
+
+	dstate_setinfo("experimental.ups.mode.buzzwords", "%s", buzzmode_buf);
+}
+
 /* similar handlers for ups.alarm */
 
 void alarm_init(void)
@@ -1579,17 +1798,22 @@ void alarm_init(void)
 #endif
 void alarm_set(const char *buf)
 {
+	/* NOTE: Differs from status_set() since we can add whole sentences
+	 *  here, not just unique tokens. Drivers are encouraged to wrap such
+	 *  sentences into brackets, especially when many alarms raised at once
+	 *  are anticipated, for readability.
+	 */
 	int ret;
-	if (strlen(alarm_buf) > 0) {
-		ret = snprintfcat(alarm_buf, sizeof(alarm_buf), " %s", buf);
+	if (strlen(alarm_buf) < 1 || (alarm_status && !strcmp(alarm_buf, "[N/A]"))) {
+		ret = snprintf(alarm_buf, sizeof(alarm_buf), "%s", buf);
 	} else {
-		ret = snprintfcat(alarm_buf, sizeof(alarm_buf), "%s", buf);
+		ret = snprintfcat(alarm_buf, sizeof(alarm_buf), " %s", buf);
 	}
 
 	if (ret < 0) {
 		/* Should we also try to print the potentially unusable buf?
 		 * Generally - likely not. But if it is short enough...
-		 * Note: LARGEBUF was the original limit mismatched vs alarm_buf
+		 * Note: LARGEBUF was the original limit mismatched vs. alarm_buf
 		 * size before PR #986.
 		 */
 		char	alarm_tmp[LARGEBUF];
@@ -1949,6 +2173,11 @@ void dstate_dump(void)
 	upsdebugx(3, "Entering %s", __func__);
 
 	node = (const st_tree_t *)dstate_getroot();
+	fflush(stderr);
 
 	dstate_tree_dump(node);
+
+	/* Make sure it lands in one piece and is logged where called */
+	fflush(stdout);
+	fflush(stderr);
 }
