@@ -3,6 +3,7 @@
    Copyright (C)
 	2003	Russell Kroll <rkroll@exploits.org>
 	2012-2017	Arnaud Quette <arnaud.quette@free.fr>
+	2020-2025	Jim Klimov <jimklimov+nut@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +33,7 @@
 
 #ifdef WIN32
 # include "wincompat.h"
-#endif
+#endif	/* WIN32 */
 
 #define DS_LISTEN_BACKLOG 16
 #define DS_MAX_READ 256		/* don't read forever from upsd */
@@ -47,13 +48,21 @@ typedef struct conn_s {
 #ifdef WIN32
 	char    buf[LARGEBUF];
 	OVERLAPPED read_overlapped;
-#endif
+#endif	/* WIN32 */
 	PCONF_CTX_t	ctx;
 	struct conn_s	*prev;
 	struct conn_s	*next;
 	int	nobroadcast;	/* connections can request to ignore send_to_all() updates */
+	int	readzero;	/* how many times in a row we had zero bytes read; see DSTATE_CONN_READZERO_THROTTLE_USEC and DSTATE_CONN_READZERO_THROTTLE_MAX */
+	int	closing;	/* raised during LOGOUT processing, to close the socket when time is right */
 } conn_t;
 
+/* sleep after read()ing zero bytes */
+#define DSTATE_CONN_READZERO_THROTTLE_USEC	500
+
+/* close socket after read()ing zero bytes this many times in a row */
+#define DSTATE_CONN_READZERO_THROTTLE_MAX	5
+
 #include "main.h"	/* for set_exit_flag(); uses conn_t itself */
 
 	extern	struct	ups_handler	upsh;
@@ -92,12 +101,23 @@ int dstate_is_stale(void);
 /* clean out the temp space for a new pass */
 void status_init(void);
 
+/* check if a status element has been set, return 0 if not, 1 if yes
+ * (considering a whole-word token in temporary status_buf) */
+int status_get(const char *buf);
+
 /* add a status element */
 void status_set(const char *buf);
 
 /* write the temporary status_buf into ups.status */
 void status_commit(void);
 
+/* similar functions for experimental.ups.mode.buzzwords, where tracked
+ * dynamically (e.g. due to ECO/ESS/HE/Smart modes supported by the device) */
+void buzzmode_init(void);
+int  buzzmode_get(const char *buf);
+void buzzmode_set(const char *buf);
+void buzzmode_commit(void);
+
 /* similar functions for ups.alarm */
 void alarm_init(void);
 void alarm_set(const char *buf);
