@@ -5,6 +5,7 @@
  * @author Copyright (C)
  *  2003 - 2007 Arnaud Quette <aquette.dev@gmail.com>
  *  2005 - 2007 Peter Selinger <selinger@users.sourceforge.net>
+ *  2021 - 2024 Jim Klimov <jimklimov+nut@gmail.com>
  *
  * This program is sponsored by MGE UPS SYSTEMS - opensource.mgeups.com
  *
@@ -34,10 +35,10 @@
 #include "nut_libusb.h"
 #ifdef WIN32
 #include "wincompat.h"
-#endif
+#endif	/* WIN32 */
 
 #define USB_DRIVER_NAME		"USB communication driver (libusb 0.1)"
-#define USB_DRIVER_VERSION	"0.47"
+#define USB_DRIVER_VERSION	"0.50"
 
 /* driver description structure */
 upsdrv_info_t comm_upsdrv_info = {
@@ -49,7 +50,6 @@ upsdrv_info_t comm_upsdrv_info = {
 };
 
 #define MAX_REPORT_SIZE         0x1800
-#define MAX_RETRY               3
 
 #if (!HAVE_STRCASESTR) && (HAVE_STRSTR && HAVE_STRLWR && HAVE_STRDUP)
 /* Only used in this file of all NUT codebase, so not in str.{c,h}
@@ -59,7 +59,7 @@ upsdrv_info_t comm_upsdrv_info = {
 static char *strcasestr(const char *haystack, const char *needle);
 #endif
 
-static void libusb_close(usb_dev_handle *udev);
+static void nut_libusb_close(usb_dev_handle *udev);
 
 /*! Add USB-related driver variables with addvar() and dstate_setinfo().
  * This removes some code duplication across the USB drivers.
@@ -154,7 +154,7 @@ static inline int matches(USBDeviceMatcher_t *matcher, USBDevice_t *device) {
  * devices from working on Mac OS X (presumably the OS is already setting
  * altinterface to 0).
  */
-static int nut_usb_set_altinterface(usb_dev_handle *udev)
+static int nut_libusb_set_altinterface(usb_dev_handle *udev)
 {
 	int altinterface = 0, ret = 0;
 	char *alt_string, *endp = NULL;
@@ -186,6 +186,20 @@ static int nut_usb_set_altinterface(usb_dev_handle *udev)
 	return ret;
 }
 
+static void nut_libusb_subdriver_defaults(usb_communication_subdriver_t *subdriver)
+{
+	if (!getval("usb_config_index"))
+		subdriver->usb_config_index = LIBUSB_DEFAULT_CONF_INDEX;
+	if (!getval("usb_hid_rep_index"))
+		subdriver->hid_rep_index = LIBUSB_DEFAULT_INTERFACE;
+	if (!getval("usb_hid_desc_index"))
+		subdriver->hid_desc_index = LIBUSB_DEFAULT_DESC_INDEX;
+	if (!getval("usb_hid_ep_in"))
+		subdriver->hid_ep_in = LIBUSB_DEFAULT_HID_EP_IN;
+	if (!getval("usb_hid_ep_out"))
+		subdriver->hid_ep_out = LIBUSB_DEFAULT_HID_EP_OUT;
+}
+
 #define usb_control_msg         typesafe_control_msg
 
 /* On success, fill in the curDevice structure and return the report
@@ -196,13 +210,15 @@ static int nut_usb_set_altinterface(usb_dev_handle *udev)
  * is accepted, or < 1 if not. If it isn't accepted, the next device
  * (if any) will be tried, until there are no more devices left.
  */
-static int libusb_open(usb_dev_handle **udevp,
+static int nut_libusb_open(usb_dev_handle **udevp,
 	USBDevice_t *curDevice, USBDeviceMatcher_t *matcher,
 	int (*callback)(usb_dev_handle *udev,
 		USBDevice_t *hd, usb_ctrl_charbuf rdbuf, usb_ctrl_charbufsize rdlen)
 	)
 {
+#ifdef HAVE_USB_DETACH_KERNEL_DRIVER_NP
 	int retries;
+#endif
 	usb_ctrl_charbufsize rdlen1, rdlen2; /* report descriptor length, method 1+2 */
 	USBDeviceMatcher_t *m;
 	struct usb_device *dev;
@@ -302,17 +318,17 @@ static int libusb_open(usb_dev_handle **udevp,
 
 #ifdef WIN32
 	busses = usb_get_busses();
-#else
+#else	/* !WIN32 */
 	/* libusb built-in; not sure why original NUT for WIN32
 	 * code differed or if it is actually better? Or why
 	 * this was not tackled in a few other files for USB?..
 	 */
 	busses = usb_busses;
-#endif
+#endif	/* !WIN32 */
 
 #ifndef __linux__ /* SUN_LIBUSB (confirmed to work on Solaris and FreeBSD) */
 	/* Causes a double free corruption in linux if device is detached! */
-	libusb_close(*udevp);
+	nut_libusb_close(*udevp);
 #endif
 
 	upsdebugx(3, "usb_busses=%p", (void*)usb_busses);
@@ -395,44 +411,32 @@ static int libusb_open(usb_dev_handle **udevp,
 #endif
 
 			if (dev->descriptor.iManufacturer) {
-				retries = MAX_RETRY;
-				while (retries > 0) {
-					ret = usb_get_string_simple(udev, dev->descriptor.iManufacturer,
-						string, sizeof(string));
-					if (ret > 0) {
-						curDevice->Vendor = xstrdup(string);
-						break;
-					}
-					retries--;
-					upsdebugx(1, "%s get iManufacturer failed, retrying...", __func__);
+				ret = nut_usb_get_string(udev, dev->descriptor.iManufacturer,
+					string, sizeof(string));
+				if (ret > 0) {
+					curDevice->Vendor = xstrdup(string);
+				} else {
+					upsdebugx(1, "%s: get Manufacturer string failed", __func__);
 				}
 			}
 
 			if (dev->descriptor.iProduct) {
-				retries = MAX_RETRY;
-				while (retries > 0) {
-					ret = usb_get_string_simple(udev, dev->descriptor.iProduct,
-						string, sizeof(string));
-					if (ret > 0) {
-						curDevice->Product = xstrdup(string);
-						break;
-					}
-					retries--;
-					upsdebugx(1, "%s get iProduct failed, retrying...", __func__);
+				ret = nut_usb_get_string(udev, dev->descriptor.iProduct,
+					string, sizeof(string));
+				if (ret > 0) {
+					curDevice->Product = xstrdup(string);
+				} else {
+					upsdebugx(1, "%s: get Product string failed", __func__);
 				}
 			}
 
 			if (dev->descriptor.iSerialNumber) {
-				retries = MAX_RETRY;
-				while (retries > 0) {
-					ret = usb_get_string_simple(udev, dev->descriptor.iSerialNumber,
-						string, sizeof(string));
-					if (ret > 0) {
-						curDevice->Serial = xstrdup(string);
-						break;
-					}
-					retries--;
-					upsdebugx(1, "%s get iSerialNumber failed, retrying...", __func__);
+				ret = nut_usb_get_string(udev, dev->descriptor.iSerialNumber,
+					string, sizeof(string));
+				if (ret > 0) {
+					curDevice->Serial = xstrdup(string);
+				} else {
+					upsdebugx(1, "%s: get Serial Number string failed", __func__);
 				}
 			}
 
@@ -488,11 +492,10 @@ static int libusb_open(usb_dev_handle **udevp,
 			/* this method requires at least libusb 0.1.8:
 			 * it force device claiming by unbinding
 			 * attached driver... From libhid */
-			retries = MAX_RETRY;
 #ifdef WIN32
 			usb_set_configuration(udev, 1);
-#endif
-
+#endif	/* WIN32 */
+			retries = 3;
 			while ((ret = usb_claim_interface(udev, usb_subdriver.hid_rep_index)) < 0) {
 				upsdebugx(2, "failed to claim USB device: %s",
 					usb_strerror());
@@ -539,8 +542,12 @@ static int libusb_open(usb_dev_handle **udevp,
 #endif
 			/* if_claimed = 1; */
 
-			nut_usb_set_altinterface(udev);
+			nut_libusb_set_altinterface(udev);
 
+			/* Did the driver provide a callback method for any further
+			 * device acceptance checks (e.g. when same ID is supported
+			 * by several sub-drivers, differing by vendor/model strings)?
+			 */
 			if (!callback) {
 				return 1;
 			}
@@ -669,7 +676,7 @@ static int libusb_open(usb_dev_handle **udevp,
 				upsdebugx(2, "Warning: report descriptor too short "
 					"(expected %" PRI_NUT_USB_CTRL_CHARBUFSIZE
 					", got %d)", rdlen, res);
-#else
+#else	/* WIN32 */
 				/* https://github.com/networkupstools/nut/issues/1690#issuecomment-1455206002 */
 				upsdebugx(0, "Warning: report descriptor too short "
 					"(expected %" PRI_NUT_USB_CTRL_CHARBUFSIZE
@@ -718,9 +725,12 @@ static int libusb_open(usb_dev_handle **udevp,
 			/* if (if_claimed)
 				usb_release_interface(udev, 0); */
 			usb_close(udev);
+			/* reset any parameters modified by unmatched drivers back to defaults */
+			nut_libusb_subdriver_defaults(&usb_subdriver);
 		}
 	}
 
+	/* If we got here, we did not return a successfully chosen device above */
 	*udevp = NULL;
 	upsdebugx(2, "libusb0: No appropriate HID device found");
 	fflush(stdout);
@@ -728,7 +738,7 @@ static int libusb_open(usb_dev_handle **udevp,
 	if (count_open_attempts == 0) {
 		upslogx(LOG_WARNING,
 			"libusb0: Could not open any HID devices: "
-			"no USB buses found");
+			"no USB buses (or devices) found");
 	}
 	else
 	if (count_open_errors > 0
@@ -737,8 +747,25 @@ static int libusb_open(usb_dev_handle **udevp,
 		upslogx(LOG_WARNING,
 			"libusb0: Could not open any HID devices: "
 			"insufficient permissions on everything");
+		if (count_open_attempts > count_open_errors) {
+			upslogx(LOG_WARNING,
+				"libusb0: except %d devices "
+				"tried but not matching the "
+				"requested criteria",
+				count_open_attempts - count_open_errors);
+		}
 	}
 
+#ifdef WIN32
+	upsdebugx(0, "Please check your Windows Device Manager: "
+		"perhaps the UPS was recognized by default OS\n"
+		"driver such as HID UPS Battery (hidbatt.sys, "
+		"hidusb.sys or similar). It could have been\n"
+		"\"restored\" by Windows Update. You can try "
+		"https://zadig.akeo.ie/ to handle it with\n"
+		"either WinUSB, libusb0.sys or libusbK.sys.");
+#endif	/* WIN32 */
+
 	return -1;
 }
 
@@ -746,7 +773,7 @@ static int libusb_open(usb_dev_handle **udevp,
  * Error handler for usb_get/set_* functions. Return value > 0 success,
  * 0 unknown or temporary failure (ignored), < 0 permanent failure (reconnect)
  */
-static int libusb_strerror(const int ret, const char *desc)
+static int nut_libusb_strerror(const int ret, const char *desc)
 {
 	if (ret > 0) {
 		return ret;
@@ -779,8 +806,9 @@ static int libusb_strerror(const int ret, const char *desc)
 # endif
 		upsdebugx(2, "%s: %s", desc, usb_strerror());
 		return 0;
-#endif	/* WIN32 */
+#endif	/* !WIN32 */
 
+	case 0: 	/** TOTHINK: Should this (probably LIBUSB_SUCCESS) be quiet? */
 	default:	/* Undetermined, log only */
 		upslogx(LOG_DEBUG, "%s: %s", desc, usb_strerror());
 		return 0;
@@ -792,10 +820,10 @@ static int libusb_strerror(const int ret, const char *desc)
  */
 
 /* Expected evaluated types for the API:
- * static int libusb_get_report(usb_dev_handle *udev,
+ * static int nut_libusb_get_report(usb_dev_handle *udev,
  *	int ReportId, unsigned char *raw_buf, int ReportSize)
  */
-static int libusb_get_report(
+static int nut_libusb_get_report(
 	usb_dev_handle *udev,
 	usb_ctrl_repindex ReportId,
 	usb_ctrl_charbuf raw_buf,
@@ -803,7 +831,7 @@ static int libusb_get_report(
 {
 	int	ret;
 
-	upsdebugx(4, "Entering libusb_get_report");
+	upsdebugx(4, "Entering nut_libusb_get_report");
 
 	if (!udev) {
 		return 0;
@@ -818,21 +846,21 @@ static int libusb_get_report(
 
 #ifdef WIN32
 	errno = -ret;
-#endif
+#endif	/* WIN32 */
 
 	/* Ignore "protocol stall" (for unsupported request) on control endpoint */
 	if (ret == -EPIPE) {
 		return 0;
 	}
 
-	return libusb_strerror(ret, __func__);
+	return nut_libusb_strerror(ret, __func__);
 }
 
 /* Expected evaluated types for the API:
- * static int libusb_set_report(usb_dev_handle *udev,
+ * static int nut_libusb_set_report(usb_dev_handle *udev,
  *	int ReportId, unsigned char *raw_buf, int ReportSize)
  */
-static int libusb_set_report(
+static int nut_libusb_set_report(
 	usb_dev_handle *udev,
 	usb_ctrl_repindex ReportId,
 	usb_ctrl_charbuf raw_buf,
@@ -853,21 +881,21 @@ static int libusb_set_report(
 
 #ifdef WIN32
 	errno = -ret;
-#endif
+#endif	/* WIN32 */
 
 	/* Ignore "protocol stall" (for unsupported request) on control endpoint */
 	if (ret == -EPIPE) {
 		return 0;
 	}
 
-	return libusb_strerror(ret, __func__);
+	return nut_libusb_strerror(ret, __func__);
 }
 
 /* Expected evaluated types for the API:
- * static int libusb_get_string(usb_dev_handle *udev,
- *	int StringIdx, char *buf, size_t buflen)
+ * static int nut_libusb_get_string(usb_dev_handle *udev,
+ *	int StringIdx, char *buf, int buflen)
  */
-static int libusb_get_string(
+static int nut_libusb_get_string(
 	usb_dev_handle *udev,
 	usb_ctrl_strindex StringIdx,
 	char *buf,
@@ -875,46 +903,43 @@ static int libusb_get_string(
 {
 	int ret;
 
-#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
-# pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
-# pragma GCC diagnostic ignored "-Wtype-limits"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
-# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE
-# pragma GCC diagnostic ignored "-Wtautological-unsigned-zero-compare"
-#endif
 	/*
-	 * usb.h:int  usb_get_string_simple(usb_dev_handle *dev, int index,
+	 * usb.h:int  nut_usb_get_string(usb_dev_handle *dev, int index,
 	 * usb.h-         char *buf, size_t buflen);
 	 */
 	if (!udev
-	|| StringIdx < 0 || (uintmax_t)StringIdx > INT_MAX
-	|| buflen < 0 || (uintmax_t)buflen > (uintmax_t)SIZE_MAX
+	|| StringIdx < 1 || StringIdx > 255
+	|| buflen < 1
 	) {
-#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
-# pragma GCC diagnostic pop
-#endif
 		return -1;
 	}
 
-	ret = usb_get_string_simple(udev, StringIdx, buf, (size_t)buflen);
+	ret = nut_usb_get_string(udev, StringIdx, buf, (size_t)buflen);
 
 #ifdef WIN32
 	errno = -ret;
-#endif
+#endif	/* WIN32 */
 
-	return libusb_strerror(ret, __func__);
+	/** 0 can be seen as an empty string, or as a success for
+	 * logging below - also tends to happen */
+	if (ret == 0) {
+		size_t len = strlen(buf);
+		upsdebugx(2, "%s: nut_usb_get_string() returned "
+			"0 (might be just success code), "
+			"actual buf length is %" PRIuSIZE, __func__, len);
+		/* if (len) */
+			return len;
+		/* else may log "nut_libusb_get_string: Success" and return 0 below */
+	}
+
+	return nut_libusb_strerror(ret, __func__);
 }
 
 /* Expected evaluated types for the API:
- * static int libusb_get_interrupt(usb_dev_handle *udev,
+ * static int nut_libusb_get_interrupt(usb_dev_handle *udev,
  *	unsigned char *buf, int bufsize, int timeout)
  */
-static int libusb_get_interrupt(
+static int nut_libusb_get_interrupt(
 	usb_dev_handle *udev,
 	usb_ctrl_charbuf buf,
 	usb_ctrl_charbufsize bufsize,
@@ -931,17 +956,17 @@ static int libusb_get_interrupt(
 
 #ifdef WIN32
 	errno = -ret;
-#endif
+#endif	/* WIN32 */
 
 	/* Clear stall condition */
 	if (ret == -EPIPE) {
 		ret = usb_clear_halt(udev, 0x81);
 	}
 
-	return libusb_strerror(ret, __func__);
+	return nut_libusb_strerror(ret, __func__);
 }
 
-static void libusb_close(usb_dev_handle *udev)
+static void nut_libusb_close(usb_dev_handle *udev)
 {
 	if (!udev) {
 		return;
@@ -988,12 +1013,12 @@ err:
 usb_communication_subdriver_t usb_subdriver = {
 	USB_DRIVER_NAME,
 	USB_DRIVER_VERSION,
-	libusb_open,
-	libusb_close,
-	libusb_get_report,
-	libusb_set_report,
-	libusb_get_string,
-	libusb_get_interrupt,
+	nut_libusb_open,
+	nut_libusb_close,
+	nut_libusb_get_report,
+	nut_libusb_set_report,
+	nut_libusb_get_string,
+	nut_libusb_get_interrupt,
 	LIBUSB_DEFAULT_CONF_INDEX,
 	LIBUSB_DEFAULT_INTERFACE,
 	LIBUSB_DEFAULT_DESC_INDEX,
