@@ -4,7 +4,7 @@
  *
  * @author Copyright (C) 2016 Eaton
  *         Copyright (C) 2016 Arnaud Quette <aquette.dev@gmail.com>
- *         Copyright (C) 2021 Jim Klimov <jimklimov+nut@gmail.com>
+ *         Copyright (C) 2021-2024 Jim Klimov <jimklimov+nut@gmail.com>
  *
  *      The logic of this file is ripped from mge-shut driver (also from
  *      Arnaud Quette), which is a "HID over serial link" UPS driver for
@@ -33,7 +33,7 @@
 #include "nut_stdint.h"
 
 #define USB_DRIVER_NAME		"USB communication driver (libusb 1.0)"
-#define USB_DRIVER_VERSION	"0.47"
+#define USB_DRIVER_VERSION	"0.50"
 
 /* driver description structure */
 upsdrv_info_t comm_upsdrv_info = {
@@ -45,7 +45,6 @@ upsdrv_info_t comm_upsdrv_info = {
 };
 
 #define MAX_REPORT_SIZE         0x1800
-#define MAX_RETRY               3
 
 static void nut_libusb_close(libusb_device_handle *udev);
 
@@ -95,7 +94,7 @@ void nut_usb_addvars(void)
 	addvar(VAR_VALUE, "usb_hid_ep_out",	"Deeper tuning of USB communications for complex devices");
 
 #ifdef LIBUSB_API_VERSION
-	dstate_setinfo("driver.version.usb", "libusb-%u.%u.%u (API: 0x%x)", v->major, v->minor, v->micro, LIBUSB_API_VERSION);
+	dstate_setinfo("driver.version.usb", "libusb-%u.%u.%u (API: 0x%08X)", v->major, v->minor, v->micro, (unsigned int)LIBUSB_API_VERSION);
 #else  /* no LIBUSB_API_VERSION */
 	dstate_setinfo("driver.version.usb", "libusb-%u.%u.%u", v->major, v->minor, v->micro);
 #endif /* LIBUSB_API_VERSION */
@@ -120,7 +119,7 @@ static inline int matches(USBDeviceMatcher_t *matcher, USBDevice_t *device) {
  * devices from working on Mac OS X (presumably the OS is already setting
  * altinterface to 0).
  */
-static int nut_usb_set_altinterface(libusb_device_handle *udev)
+static int nut_libusb_set_altinterface(libusb_device_handle *udev)
 {
 	int altinterface = 0, ret = 0;
 	char *alt_string, *endp = NULL;
@@ -153,6 +152,20 @@ static int nut_usb_set_altinterface(libusb_device_handle *udev)
 	return ret;
 }
 
+static void nut_libusb_subdriver_defaults(usb_communication_subdriver_t *subdriver)
+{
+	if (!getval("usb_config_index"))
+		subdriver->usb_config_index = LIBUSB_DEFAULT_CONF_INDEX;
+	if (!getval("usb_hid_rep_index"))
+		subdriver->hid_rep_index = LIBUSB_DEFAULT_INTERFACE;
+	if (!getval("usb_hid_desc_index"))
+		subdriver->hid_desc_index = LIBUSB_DEFAULT_DESC_INDEX;
+	if (!getval("usb_hid_ep_in"))
+		subdriver->hid_ep_in = LIBUSB_DEFAULT_HID_EP_IN;
+	if (!getval("usb_hid_ep_out"))
+		subdriver->hid_ep_out = LIBUSB_DEFAULT_HID_EP_OUT;
+}
+
 /* On success, fill in the curDevice structure and return the report
  * descriptor length. On failure, return -1.
  * Note: When callback is not NULL, the report descriptor will be
@@ -167,7 +180,9 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		USBDevice_t *hd, usb_ctrl_charbuf rdbuf, usb_ctrl_charbufsize rdlen)
 	)
 {
+#if (defined HAVE_LIBUSB_DETACH_KERNEL_DRIVER) || (defined HAVE_LIBUSB_DETACH_KERNEL_DRIVER_NP)
 	int retries;
+#endif
 	/* libusb-1.0 usb_ctrl_charbufsize is uint16_t and we
 	 * want the rdlen vars signed - so taking a wider type */
 	int32_t rdlen1, rdlen2; /* report descriptor length, method 1+2 */
@@ -190,6 +205,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 	int i;
 	int count_open_EACCESS = 0;
 	int count_open_errors = 0;
+	int count_open_attempts = 0;
 
 	/* report descriptor */
 	unsigned char	rdbuf[MAX_REPORT_SIZE];
@@ -276,7 +292,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #ifdef WIN32
 	struct usb_bus *busses;
 	busses = usb_get_busses();
-#endif
+#endif	// WIN32
  */
 
 #ifndef __linux__ /* SUN_LIBUSB (confirmed to work on Solaris and FreeBSD) */
@@ -294,8 +310,9 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		/* int		if_claimed = 0; */
 		libusb_device	*device = devlist[devnum];
 
+		count_open_attempts++;
 		libusb_get_device_descriptor(device, &dev_desc);
-		upsdebugx(2, "Checking device %" PRIuSIZE " of %" PRIuSIZE " (%04X/%04X)",
+		upsdebugx(2, "Checking device %" PRIuSIZE " of %" PRIiSIZE " (%04X/%04X)",
 			devnum + 1, devcount,
 			dev_desc.idVendor, dev_desc.idProduct);
 
@@ -390,56 +407,44 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		curDevice->bcdDevice = dev_desc.bcdDevice;
 
 		if (dev_desc.iManufacturer) {
-			retries = MAX_RETRY;
-			while (retries > 0) {
-				ret = libusb_get_string_descriptor_ascii(udev, dev_desc.iManufacturer,
-					(unsigned char*)string, sizeof(string));
-				if (ret > 0) {
-					curDevice->Vendor = strdup(string);
-					if (curDevice->Vendor == NULL) {
-						libusb_free_device_list(devlist, 1);
-						fatal_with_errno(EXIT_FAILURE, "Out of memory");
-					}
-					break;
+			ret = nut_usb_get_string(udev, dev_desc.iManufacturer,
+				string, sizeof(string));
+			if (ret > 0) {
+				curDevice->Vendor = strdup(string);
+				if (curDevice->Vendor == NULL) {
+					libusb_free_device_list(devlist, 1);
+					fatal_with_errno(EXIT_FAILURE, "Out of memory");
 				}
-				retries--;
-				upsdebugx(1, "%s get iManufacturer failed, retrying...", __func__);
+			} else {
+				upsdebugx(1, "%s: get Manufacturer string failed", __func__);
 			}
 		}
 
 		if (dev_desc.iProduct) {
-			retries = MAX_RETRY;
-			while (retries > 0) {
-				ret = libusb_get_string_descriptor_ascii(udev, dev_desc.iProduct,
-					(unsigned char*)string, sizeof(string));
-				if (ret > 0) {
-					curDevice->Product = strdup(string);
-					if (curDevice->Product == NULL) {
-						libusb_free_device_list(devlist, 1);
-						fatal_with_errno(EXIT_FAILURE, "Out of memory");
-					}
-					break;
+			ret = nut_usb_get_string(udev, dev_desc.iProduct,
+				string, sizeof(string));
+			if (ret > 0) {
+				curDevice->Product = strdup(string);
+				if (curDevice->Product == NULL) {
+					libusb_free_device_list(devlist, 1);
+					fatal_with_errno(EXIT_FAILURE, "Out of memory");
 				}
-				retries--;
-				upsdebugx(1, "%s get iProduct failed, retrying...", __func__);
+			} else {
+				upsdebugx(1, "%s: get Product string failed", __func__);
 			}
 		}
 
 		if (dev_desc.iSerialNumber) {
-			retries = MAX_RETRY;
-			while (retries > 0) {
-				ret = libusb_get_string_descriptor_ascii(udev, dev_desc.iSerialNumber,
-					(unsigned char*)string, sizeof(string));
-				if (ret > 0) {
-					curDevice->Serial = strdup(string);
-					if (curDevice->Serial == NULL) {
-						libusb_free_device_list(devlist, 1);
-						fatal_with_errno(EXIT_FAILURE, "Out of memory");
-					}
-					break;
+			ret = nut_usb_get_string(udev, dev_desc.iSerialNumber,
+				string, sizeof(string));
+			if (ret > 0) {
+				curDevice->Serial = strdup(string);
+				if (curDevice->Serial == NULL) {
+					libusb_free_device_list(devlist, 1);
+					fatal_with_errno(EXIT_FAILURE, "Out of memory");
 				}
-				retries--;
-				upsdebugx(1, "%s get iSerialNumber failed, retrying...", __func__);
+			} else {
+				upsdebugx(1, "%s: get Serial Number string failed", __func__);
 			}
 		}
 
@@ -532,12 +537,12 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #if (defined HAVE_LIBUSB_DETACH_KERNEL_DRIVER) || (defined HAVE_LIBUSB_DETACH_KERNEL_DRIVER_NP)
 		/* Then, try the explicit detach method.
 		 * This function is available on FreeBSD 10.1-10.3 */
-		retries = MAX_RETRY;
-#ifdef WIN32
+# ifdef WIN32
 		/* TODO: Align with libusb1 - initially from Windows branch made against libusb0 */
 		libusb_set_configuration(udev, 1);
-#endif
+# endif	/* WIN32 */
 
+		retries = 3;
 		while ((ret = libusb_claim_interface(udev, usb_subdriver.hid_rep_index)) != LIBUSB_SUCCESS) {
 			upsdebugx(2, "failed to claim USB device: %s",
 				libusb_strerror((enum libusb_error)ret));
@@ -599,8 +604,12 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		upsdebugx(2, "Claimed interface %d successfully",
 			usb_subdriver.hid_rep_index);
 
-		nut_usb_set_altinterface(udev);
+		nut_libusb_set_altinterface(udev);
 
+		/* Did the driver provide a callback method for any further
+		 * device acceptance checks (e.g. when same ID is supported
+		 * by several sub-drivers, differing by vendor/model strings)?
+		 */
 		if (!callback) {
 			libusb_free_config_descriptor(conf_desc);
 			libusb_free_device_list(devlist, 1);
@@ -721,8 +730,8 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
 # pragma GCC diagnostic pop
 #endif
-			upsdebugx(2, "HID descriptor too long %d (max %u)",
-				rdlen, UINT16_MAX);
+			upsdebugx(2, "HID descriptor too long %d (max %" PRIuMAX ")",
+				rdlen, (uintmax_t)UINT16_MAX);
 			goto next_device;
 		}
 
@@ -745,7 +754,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #ifndef WIN32
 			upsdebugx(2, "Warning: report descriptor too short "
 				"(expected %d, got %d)", rdlen, res);
-#else
+#else	/* WIN32 */
 			/* https://github.com/networkupstools/nut/issues/1690#issuecomment-1455206002 */
 			upsdebugx(0, "Warning: report descriptor too short "
 				"(expected %d, got %d)", rdlen, res);
@@ -802,27 +811,47 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 			/* if (if_claimed)
 				libusb_release_interface(udev, usb_subdriver.hid_rep_index); */
 			libusb_close(udev);
+			/* reset any parameters modified by unmatched drivers back to defaults */
+			nut_libusb_subdriver_defaults(&usb_subdriver);
 	}
 
+	/* If we got here, we did not return a successfully chosen device above */
 	*udevp = NULL;
 	libusb_free_device_list(devlist, 1);
 	upsdebugx(2, "libusb1: No appropriate HID device found");
 	fflush(stdout);
 
-	if (devcount < 1) {
+	if (devcount < 1 || count_open_attempts == 0) {
 		upslogx(LOG_WARNING,
 			"libusb1: Could not open any HID devices: "
-			"no USB buses found");
+			"no USB buses (or devices) found");
 	}
 	else
 	if (count_open_errors > 0
-	||  count_open_errors == count_open_EACCESS
+	&&  count_open_errors == count_open_EACCESS
 	) {
 		upslogx(LOG_WARNING,
 			"libusb1: Could not open any HID devices: "
 			"insufficient permissions on everything");
+		if (count_open_attempts > count_open_errors) {
+			upslogx(LOG_WARNING,
+				"libusb1: except %d devices "
+				"tried but not matching the "
+				"requested criteria",
+				count_open_attempts - count_open_errors);
+		}
 	}
 
+#ifdef WIN32
+	upsdebugx(0, "Please check your Windows Device Manager: "
+		"perhaps the UPS was recognized by default OS\n"
+		"driver such as HID UPS Battery (hidbatt.sys, "
+		"hidusb.sys or similar). It could have been\n"
+		"\"restored\" by Windows Update. You can try "
+		"https://zadig.akeo.ie/ to handle it with\n"
+		"either WinUSB, libusb0.sys or libusbK.sys.");
+#endif	/* WIN32 */
+
 	return -1;
 }
 
@@ -870,8 +899,9 @@ static int nut_libusb_strerror(const int ret, const char *desc)
 # endif
 		upsdebugx(2, "%s: %s", desc, libusb_strerror((enum libusb_error)ret));
 		return 0;
-#endif /* WIN32 */
+#endif	/* !WIN32 */
 
+	case LIBUSB_SUCCESS:         /** TOTHINK: Should this be quiet? */
 	case LIBUSB_ERROR_OTHER:     /** Other error */
 	default:                     /** Undetermined, log only */
 		upslogx(LOG_DEBUG, "%s: %s", desc, libusb_strerror((enum libusb_error)ret));
@@ -987,7 +1017,7 @@ static int nut_libusb_set_report(
 
 /* Expected evaluated types for the API:
  * static int nut_libusb_get_string(libusb_device_handle *udev,
- *	int StringIdx, char *buf, int buflen)
+ *	uint8_t StringIdx, unsigned char *buf, uint16_t buflen)
  */
 static int nut_libusb_get_string(
 	libusb_device_handle *udev,
@@ -997,30 +1027,24 @@ static int nut_libusb_get_string(
 {
 	int ret;
 
-#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
-# pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
-# pragma GCC diagnostic ignored "-Wtype-limits"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
-# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE
-# pragma GCC diagnostic ignored "-Wtautological-unsigned-zero-compare"
-#endif
-	if (!udev
-	|| StringIdx < 0 || (uintmax_t)StringIdx > UINT8_MAX
-	|| buflen < 0 || (uintmax_t)buflen > (uintmax_t)INT_MAX
-	) {
-#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
-# pragma GCC diagnostic pop
-#endif
+	if (!udev || StringIdx < 1 || buflen < 1) {
 		return -1;
 	}
 
-	ret = libusb_get_string_descriptor_ascii(udev, (uint8_t)StringIdx,
-		(unsigned char*)buf, (int)buflen);
+	ret = nut_usb_get_string(udev, (uint8_t)StringIdx,
+		buf, (int)buflen);
+
+	/** 0 can be seen as an empty string, or as LIBUSB_SUCCESS for
+	 * logging below - also tends to happen */
+	if (ret == 0) {
+		size_t len = strlen(buf);
+		upsdebugx(2, "%s: nut_usb_get_string() returned "
+			"0 (might be just LIBUSB_SUCCESS code), "
+			"actual buf length is %" PRIuSIZE, __func__, len);
+		/* if (len) */
+			return len;
+		/* else may log "nut_libusb_get_string: Success" and return 0 below */
+	}
 
 	return nut_libusb_strerror(ret, __func__);
 }
@@ -1086,7 +1110,7 @@ static int nut_libusb_get_interrupt(
 		) {
 			return -1;
 		}
-		ret = (usb_ctrl_charbufsize)bufsize;
+		ret = (usb_ctrl_charbufsize)tmpbufsize;
 	}
 
 	return nut_libusb_strerror(ret, __func__);
