@@ -28,7 +28,7 @@
 
 /* driver version */
 #define DRIVER_NAME	"'ATCL FOR UPS' USB driver"
-#define DRIVER_VERSION	"1.18"
+#define DRIVER_VERSION	"1.19"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -65,6 +65,9 @@ static usb_dev_handle	*udev = NULL;
 static USBDevice_t	usbdevice;
 static unsigned int	comm_failures = 0;
 
+/* Forward decls */
+static int instcmd(const char *cmdname, const char *extra);
+
 static int device_match_func(USBDevice_t *device, void *privdata)
 {
 	char *requested_vendor;
@@ -440,6 +443,9 @@ static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDev
 				case -2:
 					upsdebugx(4, "matcher: unspecified error");
 					goto next_device;
+
+				default:
+					break;
 				}
 			}
 #ifdef HAVE_LIBUSB_SET_AUTO_DETACH_KERNEL_DRIVER
@@ -535,14 +541,14 @@ void upsdrv_initups(void)
 		if (i < 3) {
 #ifdef WIN32
 			sleep(5);
-#else
+#else	/* !WIN32 */
 			if (sleep(5) == 0) {
-#endif
+#endif	/* !WIN32 */
 				usb_comm_fail("Can't open USB device, retrying ...");
 				continue;
 #ifndef WIN32
 			}
-#endif
+#endif	/* !WIN32 */
 		}
 
 		fatalx(EXIT_FAILURE,
@@ -579,6 +585,15 @@ void upsdrv_initinfo(void)
 
 	dstate_setinfo("ups.vendorid", "%04x", usbdevice.VendorID);
 	dstate_setinfo("ups.productid", "%04x", usbdevice.ProductID);
+
+	/* commands ----------------------------------------------- */
+	/* FIXME: Check with the device what our instcmd
+	 * (nee upsdrv_shutdown() contents) actually does!
+	 */
+	dstate_addcmd("shutdown.stayoff");
+
+	/* install handlers */
+	upsh.instcmd = instcmd;
 }
 
 void upsdrv_updateinfo(void)
@@ -638,44 +653,75 @@ void upsdrv_updateinfo(void)
 	status_commit();
 }
 
-/* If the UPS is on battery, it should shut down about 30 seconds after
- * receiving this packet.
- */
+/* handler for commands to be sent to UPS */
+static
+int instcmd(const char *cmdname, const char *extra)
+{
+	NUT_UNUSED_VARIABLE(extra);
+
+	/* FIXME: Which one is this - "load.off",
+	 * "shutdown.stayoff" or "shutdown.return"? */
+
+	/* Shutdown UPS */
+	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
+		/* If the UPS is on battery, it should shut down
+		 * about 30 seconds after receiving this packet.
+		 */
+
+		/* Not "const" because this mismatches arg type
+		 * of usb_interrupt_write() */
+		char	shutdown_packet[SHUTDOWN_PACKETSIZE] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+		int ret;
+
+		upslogx(LOG_DEBUG,
+			"%s: attempting to call usb_interrupt_write(01 00 00 00 00 00 00 00)",
+			__func__);
+
+		ret = usb_interrupt_write(udev,
+			SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
+			SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
+
+		if (ret <= 0) {
+			upslogx(LOG_NOTICE,
+				"%s: first usb_interrupt_write() failed: %s",
+				__func__,
+				ret ? nut_usb_strerror(ret) : "timeout");
+		}
+
+		/* Totally guessing from the .pcap file here.
+		 * TODO: configurable delay?
+		 */
+		usleep(170*1000);
+
+		ret = usb_interrupt_write(udev,
+			SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
+			SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
+
+		if (ret <= 0) {
+			upslogx(LOG_ERR,
+				"%s: second usb_interrupt_write() failed: %s",
+				__func__,
+				ret ? nut_usb_strerror(ret) : "timeout");
+		}
+
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	return STAT_INSTCMD_UNKNOWN;
+}
+
 void upsdrv_shutdown(void)
 {
-	/* Not "const" because this mismatches arg type of usb_interrupt_write() */
-	char	shutdown_packet[SHUTDOWN_PACKETSIZE] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
-	int ret;
-
-	upslogx(LOG_DEBUG,
-		"%s: attempting to call usb_interrupt_write(01 00 00 00 00 00 00 00)",
-		__func__);
-
-	ret = usb_interrupt_write(udev,
-		SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
-		SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
-
-	if (ret <= 0) {
-		upslogx(LOG_NOTICE,
-			"%s: first usb_interrupt_write() failed: %s",
-			__func__,
-			ret ? nut_usb_strerror(ret) : "timeout");
-	}
-
-	/* Totally guessing from the .pcap file here. TODO: configurable delay? */
-	usleep(170*1000);
-
-	ret = usb_interrupt_write(udev,
-		SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
-		SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
-
-	if (ret <= 0) {
-		upslogx(LOG_ERR,
-			"%s: second usb_interrupt_write() failed: %s",
-			__func__,
-			ret ? nut_usb_strerror(ret) : "timeout");
-	}
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
 
+	/* FIXME: Check with the device what our instcmd
+	 * (nee upsdrv_shutdown() contents) actually does!
+	 */
+	int	ret = do_loop_shutdown_commands("shutdown.stayoff", NULL);
+	if (handling_upsdrv_shutdown > 0)
+		set_exit_flag(ret == STAT_INSTCMD_HANDLED ? EF_EXIT_SUCCESS : EF_EXIT_FAILURE);
 }
 
 void upsdrv_help(void)
@@ -686,6 +732,7 @@ void upsdrv_makevartable(void)
 {
 	/* NOTE: This driver uses a very custom device matching method,
 	 * so does not involve nut_usb_addvars() method like others do.
+	 * When fixing, see also tools/nut-scanner/scan_usb.c "exceptions".
 	 */
 	addvar(VAR_VALUE, "vendor", "USB vendor string (or NULL if none)");
 }
