@@ -25,7 +25,7 @@
 
 #include "nutdrv_qx_voltronic-qs.h"
 
-#define VOLTRONIC_QS_VERSION "Voltronic-QS 0.07"
+#define VOLTRONIC_QS_VERSION "Voltronic-QS 0.09"
 
 /* Support functions */
 static int	voltronic_qs_claim(void);
@@ -76,7 +76,7 @@ static item_t	voltronic_qs_qx2nut[] = {
 	{ "output.voltage",		0,	NULL,	"QS\r",	"",	47,	'(',	"",	13,	17,	"%.1f",	0,	NULL,	NULL,	NULL },
 	{ "ups.load",			0,	NULL,	"QS\r",	"",	47,	'(',	"",	19,	21,	"%.0f",	0,	NULL,	NULL,	NULL },
 	{ "output.frequency",		0,	NULL,	"QS\r",	"",	47,	'(',	"",	23,	26,	"%.1f",	0,	NULL,	NULL,	NULL },
-	{ "battery.voltage",		0,	NULL,	"QS\r",	"",	47,	'(',	"",	28,	31,	"%.2f",	0,	NULL,	NULL,	NULL },
+	{ "battery.voltage",		0,	NULL,	"QS\r",	"",	47,	'(',	"",	28,	31,	"%.2f",	0,	NULL,	NULL,	qx_multiply_battvolt },
 	{ "ups.temperature",		0,	NULL,	"QS\r",	"",	47,	'(',	"",	33,	36,	"%.1f",	0,	NULL,	NULL,	NULL },
 	/* Status bits */
 	{ "ups.status",			0,	NULL,	"QS\r",	"",	47,	'(',	"",	38,	38,	NULL,	QX_FLAG_QUICK_POLL,	NULL,	NULL,	blazer_process_status_bits },	/* Utility Fail (Immediate) */
@@ -191,11 +191,19 @@ static void	voltronic_qs_initups(void)
 /* Protocol used by the UPS */
 static int	voltronic_qs_protocol(item_t *item, char *value, const size_t valuelen)
 {
-	if (strcasecmp(item->value, "V")) {
-		upsdebugx(2, "%s: invalid protocol [%s]", __func__, item->value);
-		return -1;
+	int	ret = -1;
+
+	if (!strcasecmp(item->value, "V")) {
+		upsdebugx(2, "%s: detected V protocol [%s]", __func__, item->value);
+		ret = 0;
+	} else if (!strcasecmp(item->value, "H")) {
+		upsdebugx(2, "%s: detected H protocol [%s]", __func__, item->value);
+		ret = 0;
 	}
 
+	if (ret == -1) {
+		upsdebugx(2, "%s: invalid protocol [%s]", __func__, item->value);
+	} else {
 #ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
 #pragma GCC diagnostic push
 #endif
@@ -205,12 +213,13 @@ static int	voltronic_qs_protocol(item_t *item, char *value, const size_t valuele
 #ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
 #pragma GCC diagnostic ignored "-Wformat-security"
 #endif
-	snprintf(value, valuelen, item->dfl, item->value);
+		snprintf(value, valuelen, item->dfl, item->value);
 #ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
 #pragma GCC diagnostic pop
 #endif
+	}
 
-	return 0;
+	return ret;
 }
 
 
