@@ -22,6 +22,9 @@
 #include <sys/ioctl.h>
 #include "nut_stdint.h"
 
+#define DRIVER_NAME                         "PiJuice UPS driver"
+#define DRIVER_VERSION                      "0.13"
+
 /*
  * Linux I2C userland is a bit of a mess until distros refresh to
  * the i2c-tools 4.x release that profides i2c/smbus.h for userspace
@@ -43,6 +46,9 @@
 # endif
 #endif
 
+/* Forward decls */
+static int instcmd(const char *cmdname, const char *extra);
+
 /*
  * i2c-tools pre-4.0 has a userspace header with a name that conflicts
  * with a kernel header, so it may be ignored/removed by distributions
@@ -53,7 +59,7 @@
  * situation.
  */
 #if WITH_LINUX_I2C
-#if !HAVE_DECL_I2C_SMBUS_ACCESS
+# if !HAVE_DECL_I2C_SMBUS_ACCESS
 static inline __s32 i2c_smbus_access(int file, char read_write, __u8 command,
                                      int size, union i2c_smbus_data *data)
 {
@@ -70,9 +76,9 @@ static inline __s32 i2c_smbus_access(int file, char read_write, __u8 command,
 		err = -errno;
 	return err;
 }
-#endif
+# endif
 
-#if !HAVE_DECL_I2C_SMBUS_READ_BYTE_DATA
+# if !HAVE_DECL_I2C_SMBUS_READ_BYTE_DATA
 static inline __s32 i2c_smbus_read_byte_data(int file, __u8 command)
 {
 	union i2c_smbus_data data;
@@ -84,9 +90,9 @@ static inline __s32 i2c_smbus_read_byte_data(int file, __u8 command)
 	else
 		return 0x0FF & data.byte;
 }
-#endif
+# endif
 
-#if !HAVE_DECL_I2C_SMBUS_WRITE_BYTE_DATA
+# if !HAVE_DECL_I2C_SMBUS_WRITE_BYTE_DATA
 static inline __s32 i2c_smbus_write_byte_data(int file, __u8 command, __u8 value)
 {
 	union i2c_smbus_data data;
@@ -99,9 +105,9 @@ static inline __s32 i2c_smbus_write_byte_data(int file, __u8 command, __u8 value
 	else
 		return 0x0FF & data.byte;
 }
-#endif
+# endif
 
-#if !HAVE_DECL_I2C_SMBUS_READ_WORD_DATA
+# if !HAVE_DECL_I2C_SMBUS_READ_WORD_DATA
 static inline __s32 i2c_smbus_read_word_data(int file, __u8 command)
 {
 	union i2c_smbus_data data;
@@ -113,9 +119,9 @@ static inline __s32 i2c_smbus_read_word_data(int file, __u8 command)
 	else
 		return 0x0FFFF & data.word;
 }
-#endif
+# endif
 
-#if !HAVE_DECL_I2C_SMBUS_WRITE_WORD_DATA
+# if !HAVE_DECL_I2C_SMBUS_WRITE_WORD_DATA
 static inline __s32 i2c_smbus_write_word_data(int file, __u8 command, __u16 value)
 {
 	union i2c_smbus_data data;
@@ -128,9 +134,9 @@ static inline __s32 i2c_smbus_write_word_data(int file, __u8 command, __u16 valu
 	else
 		return 0x0FFFF & data.word;
 }
-#endif
+# endif
 
-#if !HAVE_DECL_I2C_SMBUS_READ_BLOCK_DATA
+# if !HAVE_DECL_I2C_SMBUS_READ_BLOCK_DATA
 static inline __u8* i2c_smbus_read_i2c_block_data(int file, __u8 command, __u8 length, __u8 *values)
 {
 	union i2c_smbus_data data;
@@ -152,8 +158,8 @@ static inline __u8* i2c_smbus_read_i2c_block_data(int file, __u8 command, __u8 l
 
 	return values;
 }
-#endif
-#endif // if WITH_LINUX_I2C
+# endif
+#endif /* if WITH_LINUX_I2C */
 
 #define STATUS_CMD                          0x40
 #define CHARGE_LEVEL_CMD                    0x41
@@ -217,9 +223,6 @@ static inline __u8* i2c_smbus_read_i2c_block_data(int file, __u8 command, __u8 l
 #define HIGH_BATTERY_THRESHOLD              75.0
 #define NOMINAL_BATTERY_VOLTAGE             4.18
 
-#define DRIVER_NAME                         "PiJuice UPS driver"
-#define DRIVER_VERSION                      "0.11"
-
 static uint8_t i2c_address    = 0x14;
 static uint8_t shutdown_delay = 30;
 
@@ -798,6 +801,15 @@ void upsdrv_initinfo(void)
 	get_i2c_address();
 	get_battery_profile();
 	get_battery_profile_ext();
+
+	/* commands ----------------------------------------------- */
+	/* FIXME: Check with the device what our instcmd
+	 * (nee upsdrv_shutdown() contents) actually does!
+	 */
+	dstate_addcmd("shutdown.stayoff");
+
+	/* install handlers */
+	upsh.instcmd = instcmd;
 }
 
 void upsdrv_updateinfo(void)
@@ -817,9 +829,37 @@ void upsdrv_updateinfo(void)
 	dstate_dataok();
 }
 
+/* handler for commands to be sent to UPS */
+static
+int instcmd(const char *cmdname, const char *extra)
+{
+	NUT_UNUSED_VARIABLE(extra);
+
+	/* FIXME: Which one is this - "load.off",
+	 * "shutdown.stayoff" or "shutdown.return"? */
+
+	/* Shutdown UPS */
+	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
+		set_power_off();
+
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	return STAT_INSTCMD_UNKNOWN;
+}
+
 void upsdrv_shutdown(void)
 {
-	set_power_off();
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
+	/* FIXME: Check with the device what our instcmd
+	 * (nee upsdrv_shutdown() contents) actually does!
+	 */
+	int	ret = do_loop_shutdown_commands("shutdown.stayoff", NULL);
+	if (handling_upsdrv_shutdown > 0)
+		set_exit_flag(ret == STAT_INSTCMD_HANDLED ? EF_EXIT_SUCCESS : EF_EXIT_FAILURE);
 }
 
 void upsdrv_help(void)
