@@ -32,10 +32,9 @@
 
 #include "powerp-bin.h"
 #include "nut_stdint.h"
+#include "nut_float.h"
 
-#include <math.h>
-
-#define POWERPANEL_BIN_VERSION "Powerpanel-Binary 0.5"
+#define POWERPANEL_BIN_VERSION	"Powerpanel-Binary 0.61"
 
 typedef struct {
 	unsigned char	start;
