@@ -38,7 +38,7 @@
 #include "timehead.h"
 
 #define DRIVER_NAME	"Microsol Rhino UPS driver"
-#define DRIVER_VERSION	"0.54"
+#define DRIVER_VERSION	"0.55"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -464,6 +464,9 @@ CommReceive(const unsigned char *bufptr, ssize_t size)
 			break;
 		}
 
+		default:
+			break;
+
 	}
 
 	Waiting = 0;
@@ -746,13 +749,17 @@ void upsdrv_updateinfo(void)
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
 	/* basic idea: find out line status and send appropriate command */
 	/* on line: send normal shutdown, ups will return by itself on utility */
 	/* on battery: send shutdown+return, ups will cycle and return soon */
 
 	if (!SourceFail)     /* on line */
 	{
-		/* FIXME: Both legs of the if-clause send CMD_SHUT, where is the "forcing"? */
+		/* FIXME: Both legs of the if-clause send CMD_SHUT,
+		 *  where is the "forcing"? */
 		printf("On line, forcing shutdown command...\n");
 		/* send_command( CMD_SHUT ); */
 		sendshut();
