@@ -4,8 +4,8 @@
  * Documents describing the protocol implemented by this driver can be
  * found online at:
  *
- *   http://www.networkupstools.org/ups-protocols/riello/PSGPSER-0104.pdf
- *   http://www.networkupstools.org/ups-protocols/riello/PSSENTR-0100.pdf
+ *   https://www.networkupstools.org/protocols/riello/PSGPSER-0104.pdf
+ *   https://www.networkupstools.org/protocols/riello/PSSENTR-0100.pdf
  *
  * Copyright (C) 2012 - Elio Parisi <e.parisi@riello-ups.com>
  *
@@ -29,7 +29,7 @@
 #ifndef NUT_RIELLO_H_SEEN
 #define NUT_RIELLO_H_SEEN 1
 
-#include <stdint.h>
+#include "nut_stdint.h"
 
 #define CTRL_RETRIES 50
 #define CTRL_TIMEOUT 100
