@@ -31,7 +31,7 @@
 #define ENDCHAR '\r'
 
 #define DRIVER_NAME "SMS Brazil UPS driver"
-#define DRIVER_VERSION "1.01"
+#define DRIVER_VERSION "1.02"
 
 #define QUERY_SIZE 7
 #define BUFFER_SIZE 18
@@ -514,6 +514,9 @@ void upsdrv_updateinfo(void) {
 }
 
 void upsdrv_shutdown(void) {
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
     /* tell the UPS to shut down, then return - DO NOT SLEEP HERE */
     int retry;
 
@@ -531,6 +534,8 @@ void upsdrv_shutdown(void) {
     upsdebugx(2, "upsdrv Shutdown execute");
 
     for (retry = 1; retry <= MAXTRIES; retry++) {
+        /* By default, abort a previously requested shutdown
+         * (if any) and schedule a new one from this moment. */
         if (sms_instcmd("shutdown.stop", NULL) != STAT_INSTCMD_HANDLED) {
             continue;
         }
@@ -540,12 +545,14 @@ void upsdrv_shutdown(void) {
         }
 
         upslogx(LOG_ERR, "Shutting down");
-        set_exit_flag(-2); /* EXIT_SUCCESS */
+        if (handling_upsdrv_shutdown > 0)
+            set_exit_flag(EF_EXIT_SUCCESS);
         return;
     }
 
     upslogx(LOG_ERR, "Shutdown failed!");
-    set_exit_flag(-1);
+    if (handling_upsdrv_shutdown > 0)
+        set_exit_flag(EF_EXIT_FAILURE);
 }
 
 void upsdrv_help(void) {
