@@ -126,7 +126,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Tripp Lite SmartOnline driver"
-#define DRIVER_VERSION	"0.08"
+#define DRIVER_VERSION	"0.09"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -651,12 +651,12 @@ void upsdrv_initinfo(void)
 	if (get_transfer_voltage_low() && max_low_transfer) {
 		dstate_setflags("input.transfer.low", ST_FLAG_RW);
 		for (i = min_low_transfer; i <= max_low_transfer; i++)
-			dstate_addenum("input.transfer.low", "%d", i);
+			dstate_addenum("input.transfer.low", "%u", i);
 	}
 	if (get_transfer_voltage_high() && max_low_transfer) {
 		dstate_setflags("input.transfer.high", ST_FLAG_RW);
 		for (i = min_high_transfer; i <= max_high_transfer; i++)
-			dstate_addenum("input.transfer.high", "%d", i);
+			dstate_addenum("input.transfer.high", "%u", i);
 	}
 	if (get_sensitivity()) {
 		dstate_setflags("input.sensitivity", ST_FLAG_RW);
@@ -836,7 +836,7 @@ void upsdrv_updateinfo(void)
 			}
 		}
 		if (contacts_set)
-			dstate_setinfo("ups.contacts", "%02X", flags);
+			dstate_setinfo("ups.contacts", "%02X", (unsigned int)flags);
 	}
 
 	/* if we are here, status is valid */
@@ -846,7 +846,10 @@ void upsdrv_updateinfo(void)
 
 void upsdrv_shutdown(void)
 {
-	char parm[20];
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
+	char	parm[20];
 
 	if (!init_comm())
 		printf("Status failed.  Assuming it's on battery and trying a shutdown anyway.\n");
