@@ -32,8 +32,12 @@ extern "C" {
 /* callback function from read_upsconf */
 void do_upsconf_args(char *upsname, char *var, char *val);
 
-/* open the ups.conf, parse it, and call back do_upsconf_args() */
-void read_upsconf(void);
+/* open the ups.conf, parse it, and call back do_upsconf_args()
+ * returns -1 (or aborts the program) in case of errors;
+ * returns 1 if processing finished successfully
+ * See also reload_flag support in main.c for live-reload feature
+ */
+int read_upsconf(int fatal_errors);
 
 #ifdef __cplusplus
 /* *INDENT-OFF* */
