@@ -13,7 +13,7 @@
 #       is to just pick out some strings relevant for tracking config changes.
 #
 # Copyright (C) 2016-2020 Eaton
-# Copyright (C) 2020-2024 Jim Klimov <jimklimov+nut@gmail.com>
+# Copyright (C) 2020-2025 Jim Klimov <jimklimov+nut@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -109,8 +109,11 @@ fi
 ### Also can tune whether a driver "Wants" another service (would consider
 ### ordering if that one is enabled, but live if it is disabled), or if it
 ### "Requires" that (would cause that to start).
-DEPSVC_USB_SYSTEMD="systemd-udev.service systemd-udev-settle.service"
+DEPSVC_USB_SYSTEMD="systemd-udevd.service nut-udev-settle.service"
 DEPREQ_USB_SYSTEMD="Wants"
+# Serial drivers may rely on USB dongles or even custom-owned real ports:
+DEPSVC_SERIAL_SYSTEMD="systemd-udevd.service nut-udev-settle.service"
+DEPREQ_SERIAL_SYSTEMD="Wants"
 DEPSVC_NET_FULL_SYSTEMD="network-online.target systemd-resolved.service ifplugd.service"
 DEPREQ_NET_FULL_SYSTEMD="Wants"
 DEPSVC_NET_LOCAL_SYSTEMD="network.target"
@@ -172,6 +175,11 @@ if [ -z "${TIMEOUT_CMD+x}" ]; then
     fi
 fi
 
+# Should be populated by configure script during NUT build:
+NUT_WEBSITE_BASE='https://www.networkupstools.org/historic/v2.8.3'
+# Fallback, no trailing slash!
+[ -n "${NUT_WEBSITE_BASE-}" ] || NUT_WEBSITE_BASE='https://www.networkupstools.org'
+
 # Cache needed bits of ups.conf to speed up later parsing. Note that these
 # data are needed for most operations, and populated by upslist_readFile()
 UPSCONF_DATA=""
@@ -195,6 +203,7 @@ hook_validInstanceSuffixName=""
 hook_getSavedDeviceName=""
 hook_findSavedDeviceName=""
 hook_setSavedDeviceName=""
+hook_setDocLink=""
 hook_getSavedMD5=""
 hook_setSavedMD5=""
 hook_restart_upsd=""
@@ -213,6 +222,7 @@ case "${SERVICE_FRAMEWORK-}" in
         hook_getSavedDeviceName="smf_getSavedDeviceName"
         hook_findSavedDeviceName="smf_findSavedDeviceName"
         hook_setSavedDeviceName="smf_setSavedDeviceName"
+        hook_setDocLink="smf_setDocLink"
         hook_getSavedMD5="smf_getSavedMD5"
         hook_setSavedMD5="smf_setSavedMD5"
         hook_restart_upsd="smf_restart_upsd"
@@ -230,6 +240,7 @@ case "${SERVICE_FRAMEWORK-}" in
         hook_getSavedDeviceName="systemd_getSavedDeviceName"
         hook_findSavedDeviceName="systemd_findSavedDeviceName"
         hook_setSavedDeviceName="systemd_setSavedDeviceName"
+        hook_setDocLink="systemd_setDocLink"
         hook_getSavedMD5="systemd_getSavedMD5"
         hook_setSavedMD5="systemd_setSavedMD5"
         hook_restart_upsd="systemd_restart_upsd"
@@ -247,6 +258,7 @@ case "${SERVICE_FRAMEWORK-}" in
         hook_getSavedDeviceName="selftest_NOOP"
         hook_findSavedDeviceName="selftest_NOOP"
         hook_setSavedDeviceName="selftest_NOOP"
+        hook_setDocLink="selftest_NOOP"
         hook_getSavedMD5="selftest_NOOP"
         hook_setSavedMD5="selftest_NOOP"
         hook_restart_upsd="selftest_NOOP"
@@ -363,6 +375,11 @@ upslist_savednames_find_missing() {
     # Verify that all existing service units have a saved DEVICE name
     # Report those that do not have a value there (any value) so we can
     # amend those quickly after an upgrade. Otherwise we trust these.
+    # Return codes:
+    #    0) Some services were defined and at least one had DEVICE values
+    #       (those that did not are reported in stdout)
+    #    1) No services defined yet (empty stdout)
+    #    2) All service units do not have DEVICE values (all reported in stdout)
 
     # Get full instance names from system and from props
     SVCINSTS="`$hook_listInstances_raw`" && [ "${#SVCINSTS}" != 0 ] || return 1
@@ -370,14 +387,21 @@ upslist_savednames_find_missing() {
     SVCINST_PROPS="`$hook_findSavedDeviceName`" && [ "${#SVCINST_PROPS}" != 0 ] \
     || { echo $SVCINSTS ; return 2; }
 
-    # Find services which do not have saved names in props
+    # Find and report services which do *not* have saved device names in
+    # props (do not report those that have a non-trivial prop value):
+    # whether an empty value or completely absent from the list
     for SVCINST in $SVCINSTS ; do
-        echo "$SVCINST_PROPS" | grep -E "^${SVCINST_PROPS}\t"'$' >/dev/null || echo "$SVCINST"
+        echo "$SVCINST_PROPS" | grep -E "^${SVCINST}${TABCHAR}"'$' >/dev/null && echo "$SVCINST"
+        echo "$SVCINST_PROPS" | grep -E "^${SVCINST}${TABCHAR}" >/dev/null || echo "$SVCINST"
     done
+    return 0
 }
 
 upslist_savednames_find_mismatch() {
+    # NOTE: Not used currently (as of NUT v2.8.2)
     # TODO: Make use of this to fsck the enumerator configs
+    # TODO: Complete checking MD5 normalized names if original not hit
+    # TODO: Report all unit names if none has the DEVICE values? (code 2)
     #
     # Verify that all existing service units have a saved DEVICE name
     # and that such name does match the unit instance's name (original
@@ -387,20 +411,30 @@ upslist_savednames_find_mismatch() {
     # This situation might occur in some errors, but the likely case
     # is updating from versions that did not track this info yet (but
     # upslist_savednames_find_missing() should have handled those).
+    # Return codes:
+    #    0) Some services were defined and at least one had DEVICE values
+    #       (those that did not are reported in stdout)
+    #    1) No services defined yet (empty stdout)
+    #    2) All service units do not have DEVICE values (empty stdout)
 
     # Get full instance names from system and from props
     SVCINSTS="`$hook_listInstances_raw`" && [ "${#SVCINSTS}" != 0 ] || return 1
     SVCINST_PROPS="`$hook_findSavedDeviceName`" && [ "${#SVCINST_PROPS}" != 0 ] || return 2
 
-    # Find services whose props exist but services do not
+    # Find services whose props exist but services themselves do not
+    # (e.g. upgrading from some version with different naming patterns)
     echo "$SVCINST_PROPS" | while read SVCINST_PROP DEVNAME_PROP ; do
         echo "$SVCINSTS" | grep -E "^${SVCINST_PROP}"'$' >/dev/null || echo "$SVCINST_PROP"
     done
 
-    # Find services which do not have saved names in props
+    # Find and report services which do *not* have saved device names in
+    # props (do not report those that have a non-trivial prop value):
+    # whether an empty value or completely absent from the list
     for SVCINST in $SVCINSTS ; do
-        echo "$SVCINST_PROPS" | grep -E "^${SVCINST_PROP}\t"'$' >/dev/null || echo "$SVCINST"
+        echo "$SVCINST_PROPS" | grep -E "^${SVCINST}${TABCHAR}"'$' >/dev/null && echo "$SVCINST"
+        echo "$SVCINST_PROPS" | grep -E "^${SVCINST}${TABCHAR}" >/dev/null || echo "$SVCINST"
     done
+    return 0
 }
 
 upsconf_getSection_content() {
@@ -613,6 +647,7 @@ upsconf_getDriverMedia() {
                     printf '%s\n%s\n' "$CURR_DRV" "" ; return ;;
             esac
             ;;
+        # FIXME: other modbus? sysfs like INA219? GPIO? Other local devices?
         *)  printf '%s\n%s\n' "$CURR_DRV" "" ; return ;;
     esac
 }
@@ -717,6 +752,7 @@ smf_registerInstance() {
     smf_setSavedMD5 "$SVCINST" "`upsconf_getSection_MD5 "$DEVICE"`"
     # Save original device (config section) name to speed up some searches
     smf_setSavedDeviceName "$SVCINST" "$DEVICE"
+    smf_setDocLink "$SVCINST" "$DEVICE"
 
     /usr/sbin/svcadm refresh "${TARGET_FMRI}" || return
     if [ "$AUTO_START" = yes ] ; then
@@ -739,7 +775,7 @@ smf_listInstances_raw() {
 }
 smf_listInstances() {
     # Chop twice, in case there is a leading "svc:/...."
-    smf_listInstances_raw | sed -e 's/^.*://' -e 's/^.*://' | sort -n
+    smf_listInstances_raw | sed -e 's/^.*://' -e 's/^.*://' | sort -k1n -k1
 }
 smf_getSavedMD5() {
     # Query service instance $1
@@ -831,6 +867,42 @@ smf_setSavedDeviceName() {
     [ -n "$1" ] || return # No-op for global section
     smf_setSavedUniq "nut-driver:$1" "nut-driver-enumerator-generated-devicename" "DEVICE" astring "$2"
 }
+smf_setDocLink() {
+    # Save documentation links for driver of device (config section) named $2
+    # into service instance $1
+    [ -n "$1" ] || return # No-op for global section
+    __TARGET_FMRI="nut-driver:$1"
+    __DRV="`upsconf_getDriver "$2"`"
+
+### Sample:
+#tm_common_name                                           template
+#tm_common_name/C                                         ustring  "physical network interface autoconfiguration"
+#tm_doc_Network_Auto-Magic_OpenSolaris_Project_Page       template
+#tm_doc_Network_Auto-Magic_OpenSolaris_Project_Page/name  astring  "Network Auto-Magic OpenSolaris Project Page"
+#tm_doc_Network_Auto-Magic_OpenSolaris_Project_Page/uri   astring  http://hub.opensolaris.org/bin/view/Project+nwam/
+#tm_man_nwamd8                                            template
+#tm_man_nwamd8/manpath                                    astring  /usr/share/man
+#tm_man_nwamd8/section                                    astring  8
+#tm_man_nwamd8/title                                      astring  nwamd
+
+    __PG="tm_doc_${__DRV}_Page"
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" delprop "${__PG}" 2>/dev/null || true
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" addpg "${__PG}" template && \
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" setprop "${__PG}/name" = "astring:" "\"${__DRV} online\"" && \
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" setprop "${__PG}/uri" = "astring:" "${NUT_WEBSITE_BASE}/docs/man/${__DRV}.html"
+    [ $? = 0 ] && echo "OK" || { echo "FAILED to stash the service property group '${__PG}' for online docs">&2 ; return 1 ; }
+
+    __PG="tm_man_${__DRV}8"
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" delprop "${__PG}" 2>/dev/null || true
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" addpg "${__PG}" template && \
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" setprop "${__PG}/manpath" = "astring:" "/usr/local/ups/share/man" && \
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" setprop "${__PG}/section" = "astring:" "8" && \
+    /usr/sbin/svccfg -s "${__TARGET_FMRI}" setprop "${__PG}/title" = "astring:" "${__DRV}"
+    [ $? = 0 ] && echo "OK" || { echo "FAILED to stash the service property group '${__PG}' for local docs">&2 ; return 1 ; }
+
+    unset __DRV __PG __TARGET_FMRI
+    [ $? = 0 ] && echo "OK" || { echo "FAILED to stash the device doc links">&2 ; return 1 ; }
+}
 smf_restart_upsd() {
     echo "Reloading or restarting NUT data server to make sure it knows new configuration..."
     /usr/sbin/svcadm enable "nut-server" 2>/dev/null
@@ -881,8 +953,11 @@ systemd_registerInstance() {
         network)
             DEPSVC="$DEPSVC_NET_FULL_SYSTEMD"
             DEPREQ="$DEPREQ_NET_FULL_SYSTEMD" ;;
-        serial) ;;
+        serial)
+            DEPSVC="$DEPSVC_SERIAL_SYSTEMD"
+            DEPREQ="$DEPREQ_SERIAL_SYSTEMD" ;;
         '') ;;
+        # FIXME: modbus? sysfs like INA219? GPIO? Other local devices?
         *)  echo "WARNING: Unexpected NUT media type ignored: '${_MED}'" >&2 ;;
     esac
     if [ -n "$DEPSVC" ]; then
@@ -902,6 +977,7 @@ EOF
 
     systemd_setSavedMD5 "$SVCINST" "`upsconf_getSection_MD5 "$DEVICE"`"
     systemd_setSavedDeviceName "$SVCINST" "$DEVICE"
+    systemd_setDocLink "$SVCINST" "$DEVICE"
 
     if [ "$AUTO_START" = yes ] ; then
         systemd_refreshSupervizor || echo "WARNING: Somehow managed to fail systemd_refreshSupervizor()" >&2
@@ -926,7 +1002,7 @@ systemd_listInstances_raw() {
     /bin/systemctl show 'nut-driver@*' -p Id | grep -E '=nut-driver' | sed 's,^Id=,,'
 }
 systemd_listInstances() {
-    systemd_listInstances_raw | sed -e 's/^.*@//' -e 's/\.service$//' | sort -n
+    systemd_listInstances_raw | sed -e 's/^.*@//' -e 's/\.service$//' | sort -k1n -k1
 }
 systemd_getSavedMD5() {
     # Query service instance $1 or global section
@@ -972,6 +1048,21 @@ Environment='DEVICE="$2"'
 EOF
     [ $? = 0 ] && echo "OK" || { echo "FAILED to stash the device name">&2 ; return 1 ; }
 }
+systemd_setDocLink() {
+    # Save documentation links for driver of device (config section) named $2
+    # into service instance $1
+    [ -n "$1" ] || return # No-op for global section
+    PROPFILE="${SYSTEMD_CONFPATH}/nut-driver@$1.service.d/nut-driver-enumerator-generated-doclink.conf"
+    mkdir -p "${SYSTEMD_CONFPATH}/nut-driver@$1.service.d" && \
+    __DRV="`upsconf_getDriver "$2"`"
+    cat > "${PROPFILE}" << EOF
+[Unit]
+Documentation=man:${__DRV}(8)
+Documentation=${NUT_WEBSITE_BASE}/docs/man/${__DRV}.html
+EOF
+    unset __DRV
+    [ $? = 0 ] && echo "OK" || { echo "FAILED to stash the device name">&2 ; return 1 ; }
+}
 systemd_setSavedMD5() {
     # Save checksum value $2 into service instance $1
     PROP="SECTION_CHECKSUM"
@@ -1097,7 +1188,7 @@ upslist_readFile() {
 
     if [ "${#UPSCONF_DATA}" != 0 ] ; then
         # Note that section-name brackets should contain a single token
-        UPSLIST_FILE="$(echo "$UPSCONF_DATA_SDP" | grep -E '^\[[^'"$TABCHAR"'\ ]*\]$' | sed 's,^\[\(.*\)\]$,\1,' | sort -n)" \
+        UPSLIST_FILE="$(echo "$UPSCONF_DATA_SDP" | grep -E '^\[[^'"$TABCHAR"'\ ]*\]$' | sed 's,^\[\(.*\)\]$,\1,' | sort -k1n -k1)" \
             || UPSLIST_FILE=""
         if [ "${#UPSLIST_FILE}" = 0 ] ; then
             echo "Error reading the '$UPSCONF' file or it does not declare any device configurations: no section declarations in parsed normalized contents" >&2
@@ -1210,32 +1301,54 @@ nut_driver_enumerator_main() {
     # while the check for no new device section definitions is just boolean.
     # We can only exit quickly if both there are no changed sections and no
     # new or removed sections since last run.
-    ( [ -z "$UPSLIST_FILE" -a -z "$UPSLIST_SVCS" ] || ( \
+    { [ -z "$UPSLIST_FILE" -a -z "$UPSLIST_SVCS" ] || { \
         NEW_CHECKSUM="`upslist_checksums_unchanged "$UPSLIST_FILE" "$UPSLIST_SVCS"`" \
         && [ "${#NEW_CHECKSUM}" = 0 ] \
-        && upslist_equals "$UPSLIST_FILE" "$UPSLIST_SVCS" \
-    ) ) \
+        && upslist_equals "$UPSLIST_FILE" "$UPSLIST_SVCS" ; \
+    } ; } \
     && if [ -z "$DAEMON_SLEEP" -o "${VERBOSE_LOOP}" = yes ] ; then \
         echo "`date -u` : OK: No changes to reconcile between ${SERVICE_FRAMEWORK} service instances and device configurations in '$UPSCONF'" ; \
        fi \
     && [ "$RESTART_ALL" = no ] && return 0
 
-    if [ "${#NEW_CHECKSUM}" != 0 ]; then
+    # Check list of devices with new section contents (per checksum) as
+    # compared to older runs (stashed in service instance configurations).
+    if [ "${#NEW_CHECKSUM}" = 0 ]; then
+        if [ "${VERBOSE_LOOP}" = yes ] ; then
+            echo "`date -u` : No changes to reconcile between *contents* of ${SERVICE_FRAMEWORK} service instances and device configurations in '$UPSCONF', but the *list* of instances vs. devices seems to have changed"
+        fi
+    else
+        if [ "${VERBOSE_LOOP}" = yes ] ; then
+            echo "`date -u` : Got some changes to reconcile between ${SERVICE_FRAMEWORK} service instances and device configurations in '$UPSCONF', content checksums changed for: `echo ${NEW_CHECKSUM} | tr '\n' ' '`"
+        fi
         for UPSS in $NEW_CHECKSUM ; do
-            CURR_DRV="`upsconf_getDriver "$UPSS"`" || CURR_DRV=""
+            # NOTE: Pedantically, UPSS is a service instance name,
+            # and may be the MD5-normalized version in certain cases.
+            # For some operations below we need the original ups.conf
+            # section name for the device - the CURR_DEV value.
+            CURR_DEV="`USE_SAVEDINST=true get_device_for_service "${UPSS}"`" && [ "${#CURR_DEV}" -gt 0 ] || CURR_DEV="${UPSS}"
+            CURR_DRV="`upsconf_getDriver "${CURR_DEV}"`" || CURR_DRV=""
             DO_UNREGISTER=yes
             if [ -n "$CURR_DRV" ] ; then
                 # If reload is handled and does not complain,
                 # we are OK to proceed without re-defining
                 # and restarting the driver service instance.
-                "/usr/local/ups/bin/$CURR_DRV" -a "$UPSS" -c reload-or-error >/dev/null 2>/dev/null \
+                if [ "${VERBOSE_LOOP}" = yes ] ; then
+                    echo "`date -u` : Reloading ${SERVICE_FRAMEWORK} service instance '${UPSS}' whose section '${CURR_DEV}' in config file has changed: calling driver program '${CURR_DRV}' for the low-level work..." >&2
+                fi
+                "/usr/local/ups/bin/$CURR_DRV" -a "${CURR_DEV}" -c reload-or-error >/dev/null 2>/dev/null \
+                || { if [ "${VERBOSE_LOOP}" = yes ] ; then sleep 1; "/usr/local/ups/bin/$CURR_DRV" -a "${CURR_DEV}" -c reload-or-error >&2 ; fi; } \
                 && DO_UNREGISTER=no
             fi
             if [ "$DO_UNREGISTER" = yes ] ; then
-                echo "Dropping old ${SERVICE_FRAMEWORK} service instance ${UPSS} whose section in config file has changed..." >&2
+                echo "Dropping old ${SERVICE_FRAMEWORK} service instance '${UPSS}' whose section '${CURR_DEV}' in config file has changed..." >&2
                 $hook_unregisterInstance "$UPSS"
+                # Re-registration to reconcile below should set the "saved" values
             else
-                echo "Keeping ${SERVICE_FRAMEWORK} service instance ${UPSS} whose section in config file has changed: live reload sufficed" >&2
+                echo "Keeping ${SERVICE_FRAMEWORK} service instance '${UPSS}' whose section '${CURR_DEV}' in config file has changed: live reload sufficed. Saving updated info into service properties." >&2
+                $hook_setSavedMD5 "$UPSS" "`upsconf_getSection_MD5 "${CURR_DEV}"`"
+                # TOTHINK: This is already there, else we redefine units for bigger discrepancies?
+                # $hook_setSavedDeviceName "$UPSS" "${CURR_DEV}"
             fi
         done
         upslist_readSvcs "after updating for new config section checksums"
@@ -1688,6 +1801,11 @@ $0 --show-device-config-value DEV KEY [KEY...]
         If several keys or flags are requested, their values are reported
         one per line in the same order (including empty lines for missing
         values); any missing value yields a non-zero exit code.
+
+For more information please Read The Fine Manual ('man nut-driver-enumerator')
+and/or see
+        ${NUT_WEBSITE_BASE}/docs/man/nut-driver-enumerator.html
+Also check documentation of ups.conf and upsdrvsvcctl
 EOF
 }
 
