@@ -9,8 +9,8 @@
  *  https://github.com/networkupstools/nut/issues/1023
  *
  * Copyright (C)
- *      2021    Nick Briggs <nicholas.h.briggs@gmail.com>
- *      2022    Jim Klimov <jimklimov+nut@gmail.com>
+ *      2021         Nick Briggs <nicholas.h.briggs@gmail.com>
+ *      2022-2024    Jim Klimov <jimklimov+nut@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +61,9 @@ static void PrintBufAndData(uint8_t *buf, size_t bufSize, HIDData_t *pData) {
 	}
 	printf("%02x\"", buf[bufSize - 1]);
 	printf(" offset %u size %u logmin %ld (0x%lx) logmax %ld (0x%lx)",
-		pData->Offset, pData->Size, pData->LogMin, pData->LogMin, pData->LogMax, pData->LogMax);
+		pData->Offset, pData->Size,
+		pData->LogMin, (unsigned long)pData->LogMin,
+		pData->LogMax, (unsigned long)pData->LogMax);
 }
 
 static int RunBuiltInTests(char *argv[]) {
@@ -123,7 +125,7 @@ static int RunBuiltInTests(char *argv[]) {
 
 		GetValue(reportBuf, &data, &value);
 
-		printf("Test #%" PRIiSIZE " ", i + 1);
+		printf("Test #%" PRIuSIZE " ", i + 1);
 		PrintBufAndData(reportBuf, bufSize,  &data);
 		if (value == testData[i].expectedValue) {
 			printf(" value %ld PASS\n", value);
@@ -153,7 +155,7 @@ static int RunBuiltInTests(char *argv[]) {
 	bufU[1] = (unsigned char)0x01;
 
 	/* Check different conversion methods and hope current build CPU,
-	 * C implementation etc. do not mess up bit-shifting vs rotation,
+	 * C implementation etc. do not mess up bit-shifting vs. rotation,
 	 * zeroing high bits, int type width extension et al. If something
 	 * is mismatched below, the production NUT code may need adaptations
 	 * for that platform to count stuff correctly!
