@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2011-2016 - EATON
+ *  Copyright (C) 2020-2024 - Jim Klimov <jimklimov+nut@gmail.com> - support and modernization of codebase
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +26,11 @@
 #include "common.h"
 #include "nut-scan.h"
 
+/* externally visible to nutscan-init */
+int nutscan_unload_usb_library(void);
+
 #ifdef WITH_USB
+
 #include "upsclient.h"
 #include "nutscan-usb.h"
 #include <stdio.h>
@@ -35,17 +40,25 @@
 /* dynamic link library stuff */
 static lt_dlhandle dl_handle = NULL;
 static const char *dl_error = NULL;
-static int (*nut_usb_close)(libusb_device_handle *dev);
-static int (*nut_usb_get_string_simple)(libusb_device_handle *dev, int index,
-		char *buf, size_t buflen);
+static char *dl_saved_libname = NULL;
 
+static int (*nut_usb_close)(libusb_device_handle *dev);
+static int (*nut_usb_control_transfer)(libusb_device_handle *dev,
+		uint8_t request_type, uint8_t bRequest, uint16_t wValue, uint16_t wIndex,
+		unsigned char *data, uint16_t wLength, unsigned int timeout);
+static int (*nut_usb_get_string_with_langid)(libusb_device_handle *dev, int index, int langid,
+		char *buf, size_t buflen);
+/* Fallback implem if the above is not a library symbol */
+static int nut_usb_get_string_with_langid_control_transfer(
+		libusb_device_handle *dev, int index, int langid,
+		char *buf, size_t buflen);
 
 /* Compatibility layer between libusb 0.1 and 1.0 */
 #if WITH_LIBUSB_1_0
- #define USB_INIT_SYMBOL "libusb_init"
- #define USB_OPEN_SYMBOL "libusb_open"
- #define USB_CLOSE_SYMBOL "libusb_close"
- #define USB_STRERROR_SYMBOL "libusb_strerror"
+# define USB_INIT_SYMBOL "libusb_init"
+# define USB_OPEN_SYMBOL "libusb_open"
+# define USB_CLOSE_SYMBOL "libusb_close"
+# define USB_STRERROR_SYMBOL "libusb_strerror"
  static int (*nut_usb_open)(libusb_device *dev, libusb_device_handle **handle);
  static int (*nut_usb_init)(libusb_context **ctx);
  static void (*nut_usb_exit)(libusb_context *ctx);
@@ -57,11 +70,14 @@ static int (*nut_usb_get_string_simple)(libusb_device_handle *dev, int index,
  static uint8_t (*nut_usb_get_port_number)(libusb_device *dev);
  static int (*nut_usb_get_device_descriptor)(libusb_device *dev,
 	struct libusb_device_descriptor *desc);
+# define USB_DT_STRING	LIBUSB_DT_STRING
+# define USB_ENDPOINT_IN	LIBUSB_ENDPOINT_IN
+# define USB_REQ_GET_DESCRIPTOR	LIBUSB_REQUEST_GET_DESCRIPTOR
 #else /* => WITH_LIBUSB_0_1 */
- #define USB_INIT_SYMBOL "usb_init"
- #define USB_OPEN_SYMBOL "usb_open"
- #define USB_CLOSE_SYMBOL "usb_close"
- #define USB_STRERROR_SYMBOL "usb_strerror"
+# define USB_INIT_SYMBOL "usb_init"
+# define USB_OPEN_SYMBOL "usb_open"
+# define USB_CLOSE_SYMBOL "usb_close"
+# define USB_STRERROR_SYMBOL "usb_strerror"
  static libusb_device_handle * (*nut_usb_open)(struct usb_device *dev);
  static void (*nut_usb_init)(void);
  static int (*nut_usb_find_busses)(void);
@@ -74,7 +90,16 @@ static int (*nut_usb_get_string_simple)(libusb_device_handle *dev, int index,
  static char * (*nut_usb_strerror)(void);
 #endif /* WITH_LIBUSB_1_0 */
 
-/* return 0 on error; visible externally */
+/* Return 0 on success, -1 on error e.g. "was not loaded";
+ * other values may be possible if lt_dlclose() errors set them;
+ * visible externally */
+int nutscan_unload_library(int *avail, lt_dlhandle *pdl_handle, char **libpath);
+int nutscan_unload_usb_library(void)
+{
+	return nutscan_unload_library(&nutscan_avail_usb, &dl_handle, &dl_saved_libname);
+}
+
+/* Return 0 on error; visible externally */
 int nutscan_load_usb_library(const char *libname_path);
 int nutscan_load_usb_library(const char *libname_path)
 {
@@ -88,12 +113,12 @@ int nutscan_load_usb_library(const char *libname_path)
 	}
 
 	if (libname_path == NULL) {
-		fprintf(stderr, "USB library not found. USB search disabled.\n");
+		upsdebugx(0, "USB library not found. USB search disabled.");
 		return 0;
 	}
 
 	if (lt_dlinit() != 0) {
-		fprintf(stderr, "Error initializing lt_init\n");
+		upsdebugx(0, "%s: Error initializing lt_dlinit", __func__);
 		return 0;
 	}
 
@@ -102,7 +127,9 @@ int nutscan_load_usb_library(const char *libname_path)
 			dl_error = lt_dlerror();
 			goto err;
 	}
-	lt_dlerror();      /* Clear any existing error */
+
+	/* Clear any existing error */
+	lt_dlerror();
 
 	*(void **) (&nut_usb_init) = lt_dlsym(dl_handle, USB_INIT_SYMBOL);
 	if ((dl_error = lt_dlerror()) != NULL) {
@@ -165,10 +192,10 @@ int nutscan_load_usb_library(const char *libname_path)
 	*(void **) (&nut_usb_get_port_number) = lt_dlsym(dl_handle,
 					"libusb_get_port_number");
 	if ((dl_error = lt_dlerror()) != NULL) {
-			fprintf(stderr,
+			upsdebugx(0, "WARNING: %s: "
 				"While loading USB library (%s), failed to find libusb_get_port_number() : %s. "
-				"The \"busport\" USB matching option will be disabled.\n",
-				libname_path, dl_error);
+				"The \"busport\" USB matching option will be disabled.",
+				__func__, libname_path, dl_error);
 			nut_usb_get_port_number = NULL;
 	}
 
@@ -178,11 +205,20 @@ int nutscan_load_usb_library(const char *libname_path)
 			goto err;
 	}
 
-	*(void **) (&nut_usb_get_string_simple) = lt_dlsym(dl_handle,
-					"libusb_get_string_descriptor_ascii");
+	*(void **) (&nut_usb_control_transfer) = lt_dlsym(dl_handle,
+					"libusb_control_transfer");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
+
+	*(void **) (&nut_usb_get_string_with_langid) = lt_dlsym(dl_handle,
+					"libusb_get_string_descriptor");
+	if ((dl_error = lt_dlerror()) != NULL) {
+			/* This one may be only defined in a header as an inline method;
+			 * then we are adapting it via nut_usb_control_transfer().
+			 */
+			nut_usb_get_string_with_langid = NULL;
+	}
 #else /* for libusb 0.1 */
 	*(void **) (&nut_usb_find_busses) = lt_dlsym(dl_handle,
 					"usb_find_busses");
@@ -210,21 +246,40 @@ int nutscan_load_usb_library(const char *libname_path)
 			goto err;
 	}
 
-	*(void **) (&nut_usb_get_string_simple) = lt_dlsym(dl_handle,
-					"usb_get_string_simple");
+	*(void **) (&nut_usb_control_transfer) = lt_dlsym(dl_handle,
+					"usb_control_msg");
 	if ((dl_error = lt_dlerror()) != NULL) {
 			goto err;
 	}
-#endif /* WITH_LIBUSB_1_0 */
+
+	*(void **) (&nut_usb_get_string_with_langid) = lt_dlsym(dl_handle,
+					"usb_get_string");
+	if ((dl_error = lt_dlerror()) != NULL) {
+			/* See comment above */
+			nut_usb_get_string_with_langid = NULL;
+	}
+#endif /* not WITH_LIBUSB_1_0 => for libusb 0.1 */
+
+	if (nut_usb_get_string_with_langid == NULL) {
+			nut_usb_get_string_with_langid = nut_usb_get_string_with_langid_control_transfer;
+	}
+
+	if (dl_saved_libname)
+		free(dl_saved_libname);
+	dl_saved_libname = xstrdup(libname_path);
 
 	return 1;
 
 err:
-	fprintf(stderr,
-		"Cannot load USB library (%s) : %s. USB search disabled.\n",
+	upsdebugx(0,
+		"Cannot load USB library (%s) : %s. USB search disabled.",
 		libname_path, dl_error);
 	dl_handle = (void *)1;
 	lt_dlexit();
+	if (dl_saved_libname) {
+		free(dl_saved_libname);
+		dl_saved_libname = NULL;
+	}
 	return 0;
 }
 /* end of dynamic link library stuff */
@@ -247,6 +302,107 @@ static char* is_usb_device_supported(usb_device_id_t *usb_device_id_list,
 	return NULL;
 }
 
+/* This one may be only defined in a header as an inline method;
+ * then we are adapting it via nut_usb_control_transfer() per
+ * https://github.com/libusb/libusb-compat-0.1/blob/eaed7b8f11badaf07a91e07538f6e8842f59eaab/libusb/libusb-dload.h#L165-L171
+ */
+static int nut_usb_get_string_with_langid_control_transfer(
+		libusb_device_handle *dev, int index, int langid,
+		char *buf, size_t buflen)
+{
+	return (*nut_usb_control_transfer)(
+		dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
+		(USB_DT_STRING << 8) + index, langid,
+		(unsigned char *)buf, buflen, 1000);
+}
+
+/* Replicated from usb-common.c with consideration for nut-scanner's
+ * use of method pointers instead of direct linking. API neutral,
+ * handles retries. Retries were originally introduced for "Tripp Lite"
+ * devices, see https://github.com/networkupstools/nut/issues/414
+ */
+#define MAX_STRING_DESC_TRIES 3
+
+static int nut_usb_get_string_descriptor(
+	libusb_device_handle *udev,
+	int StringIdx,
+	int langid,
+	char *buf,
+	size_t buflen)
+{
+	int ret = -1;
+	int tries = MAX_STRING_DESC_TRIES;
+
+	while (tries--) {
+		ret = (*nut_usb_get_string_with_langid)(
+			udev, StringIdx,
+			langid, buf, buflen);
+		if (ret >= 0) {
+			break;
+		} else if (tries) {
+			upsdebugx(1, "%s: string descriptor %d request failed, retrying...", __func__, StringIdx);
+			usleep(50000);	/* 50 ms, might help in some cases */
+		}
+	}
+	return ret;
+}
+
+/* Replicated from usb-common.c with consideration for nut-scanner's
+ * use of method pointers instead of direct linking. API neutral,
+ * assumes en_US if langid descriptor is broken */
+static int nut_usb_get_string(
+	libusb_device_handle *udev,
+	int StringIdx,
+	char *buf,
+	size_t buflen)
+{
+	int ret;
+	char buffer[255];
+	int langid;
+	int len;
+	int i;
+
+	if (!udev || StringIdx < 1 || StringIdx > 255) {
+		return -1;
+	}
+
+	/* request langid descriptor */
+	ret = nut_usb_get_string_descriptor(udev, 0, 0, buffer, 4);
+	if (ret < 0)
+		return ret;
+
+	if (ret == 4 && buffer[0] >= 4 && buffer[1] == USB_DT_STRING) {
+		langid = buffer[2] | (buffer[3] << 8);
+	} else {
+		upsdebugx(1, "%s: Broken language identifier, assuming en_US", __func__);
+		langid = 0x0409;
+	}
+
+	/* retrieve string in preferred language */
+	ret = nut_usb_get_string_descriptor(udev, StringIdx, langid, buffer, sizeof(buffer));
+	if (ret < 0) {
+#ifdef WIN32
+		/* FIXME NUT_WIN32_INCOMPLETE? : only for libusb0 ? */
+		errno = -ret;
+#endif	/* WIN32 */
+		return ret;
+	}
+
+	/* translate simple UTF-16LE to 8-bit */
+	len = ret < (int)buflen ? ret : (int)buflen;
+	len = len / 2 - 1;	/* 16-bit characters, without header */
+	len = len < (int)buflen - 1 ? len : (int)buflen - 1;	/* reserve for null terminator */
+	for (i = 0; i < len; i++) {
+		if (buffer[2 + i * 2 + 1] == 0)
+			buf[i] = buffer[2 + i * 2];
+		else
+			buf[i] = '?';	/* not decoded */
+	}
+	buf[i] = '\0';
+
+	return len;
+}
+
 /* return NULL if error */
 nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 {
@@ -329,7 +485,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 #if WITH_LIBUSB_1_0
 	if ((*nut_usb_init)(NULL) < 0) {
 		(*nut_usb_exit)(NULL);
-		fatal_with_errno(EXIT_FAILURE, "Failed to init libusb 1.0");
+		upsdebug_with_errno(0, "Failed to init libusb 1.0");
+		/* nutscan_avail_usb = 0; */
+		return NULL;
 	}
 #else  /* => WITH_LIBUSB_0_1 */
 	(*nut_usb_init)();
@@ -341,7 +499,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 	devcount = (*nut_usb_get_device_list)(NULL, &devlist);
 	if (devcount <= 0) {
 		(*nut_usb_exit)(NULL);
-		fatal_with_errno(EXIT_FAILURE, "No USB device found");
+		upsdebug_with_errno(0, "No USB device found");
+		/* nutscan_avail_usb = 0; */
+		return NULL;
 	}
 
 	for (i = 0; i < devcount; i++) {
@@ -361,7 +521,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 		if (busname == NULL) {
 			(*nut_usb_free_device_list)(devlist, 1);
 			(*nut_usb_exit)(NULL);
-			fatal_with_errno(EXIT_FAILURE, "Out of memory");
+			upsdebug_with_errno(0, "%s: Out of memory", __func__);
+			/* nutscan_avail_usb = 0; */
+			return NULL;
 		}
 		snprintf(busname, 4, "%03d", bus_num);
 
@@ -369,7 +531,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 		if (device_port == NULL) {
 			(*nut_usb_free_device_list)(devlist, 1);
 			(*nut_usb_exit)(NULL);
-			fatal_with_errno(EXIT_FAILURE, "Out of memory");
+			upsdebug_with_errno(0, "%s: Out of memory", __func__);
+			/* nutscan_avail_usb = 0; */
+			return NULL;
 		} else {
 			uint8_t device_addr = (*nut_usb_get_device_address)(dev);
 			if (device_addr > 0) {
@@ -384,7 +548,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 			if (bus_port == NULL) {
 				(*nut_usb_free_device_list)(devlist, 1);
 				(*nut_usb_exit)(NULL);
-				fatal_with_errno(EXIT_FAILURE, "Out of memory");
+				upsdebug_with_errno(0, "%s: Out of memory", __func__);
+				/* nutscan_avail_usb = 0; */
+				return NULL;
 			} else {
 				uint8_t port_num = (*nut_usb_get_port_number)(dev);
 				if (port_num > 0) {
@@ -422,9 +588,12 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 #if WITH_LIBUSB_1_0
 				ret = (*nut_usb_open)(dev, &udev);
 				if (!udev || ret != LIBUSB_SUCCESS) {
-					fprintf(stderr, "Failed to open device "
-						"bus '%s' device/port '%s' bus/port '%s', skipping: %s\n",
-						busname, device_port, bus_port,
+					upsdebugx(0, "WARNING: %s: "
+						"Failed to open device "
+						"bus '%s' device/port '%s' "
+						"bus/port '%s', skipping: %s",
+						__func__, busname,
+						device_port, bus_port,
 						(*nut_usb_strerror)(ret));
 
 					/* Note: closing is not applicable
@@ -446,9 +615,10 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 				udev = (*nut_usb_open)(dev);
 				if (!udev) {
 					/* TOTHINK: any errno or similar to test? */
-					fprintf(stderr, "Failed to open device "
-						"bus '%s' device/port '%s', skipping: %s\n",
-						busname, device_port,
+					upsdebugx(0, "WARNING: %s: "
+						"Failed to open device "
+						"bus '%s' device/port '%s', skipping: %s",
+						__func__, busname, device_port,
 						(*nut_usb_strerror)());
 					continue;
 				}
@@ -456,7 +626,7 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 
 				/* get serial number */
 				if (iSerialNumber) {
-					ret = (*nut_usb_get_string_simple)(udev,
+					ret = nut_usb_get_string(udev,
 						iSerialNumber, string, sizeof(string));
 					if (ret > 0) {
 						serialnumber = strdup(str_rtrim(string, ' '));
@@ -472,14 +642,16 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
-							fatal_with_errno(EXIT_FAILURE, "Out of memory");
+							upsdebug_with_errno(0, "%s: Out of memory", __func__);
+							/* nutscan_avail_usb = 0; */
+							return NULL;
 						}
 					}
 				}
 
 				/* get product name */
 				if (iProduct) {
-					ret = (*nut_usb_get_string_simple)(udev,
+					ret = nut_usb_get_string(udev,
 						iProduct, string, sizeof(string));
 					if (ret > 0) {
 						device_name = strdup(str_rtrim(string, ' '));
@@ -496,14 +668,16 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
-							fatal_with_errno(EXIT_FAILURE, "Out of memory");
+							upsdebug_with_errno(0, "%s: Out of memory", __func__);
+							/* nutscan_avail_usb = 0; */
+							return NULL;
 						}
 					}
 				}
 
 				/* get vendor name */
 				if (iManufacturer) {
-					ret = (*nut_usb_get_string_simple)(udev,
+					ret = nut_usb_get_string(udev,
 						iManufacturer, string, sizeof(string));
 					if (ret > 0) {
 						vendor_name = strdup(str_rtrim(string, ' '));
@@ -521,15 +695,16 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 							(*nut_usb_free_device_list)(devlist, 1);
 							(*nut_usb_exit)(NULL);
 #endif	/* WITH_LIBUSB_1_0 */
-							fatal_with_errno(EXIT_FAILURE, "Out of memory");
+							upsdebug_with_errno(0, "%s: Out of memory", __func__);
+							/* nutscan_avail_usb = 0; */
+							return NULL;
 						}
 					}
 				}
 
 				nut_dev = nutscan_new_device();
 				if (nut_dev == NULL) {
-					fprintf(stderr,
-						"Memory allocation error\n");
+					upsdebugx(0, "%s: Memory allocation error", __func__);
 					nutscan_free_device(current_nut_dev);
 					free(serialnumber);
 					free(device_name);
@@ -557,36 +732,87 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 				}
 				nut_dev->port = strdup("auto");
 
-				sprintf(string, "%04X", VendorID);
-				nutscan_add_option_to_device(nut_dev,
-					"vendorid",
-					string);
-
-				sprintf(string, "%04X", ProductID);
-				nutscan_add_option_to_device(nut_dev,
-					"productid",
-					string);
-
-				if (device_name) {
+				/* FIXME: https://github.com/networkupstools/nut/pull/1763
+				 * and linked issues: some drivers do not use
+				 * the common nut_usb_addvars() method, and
+				 * do not otherwise support these options:
+				 */
+				if (strncmp(driver_name, "bcmxcp_usb", 10)
+				 && strncmp(driver_name, "richcomm_usb", 12)
+				 && strncmp(driver_name, "nutdrv_atcl_usb", 15)
+				) {
+					sprintf(string, "%04X", VendorID);
 					nutscan_add_option_to_device(nut_dev,
-						"product",
-						device_name);
-					free(device_name);
-					device_name = NULL;
-				}
+						"vendorid",
+						string);
 
-				if (serialnumber) {
+					sprintf(string, "%04X", ProductID);
 					nutscan_add_option_to_device(nut_dev,
-						"serial",
-						serialnumber);
-					free(serialnumber);
-					serialnumber = NULL;
+						"productid",
+						string);
+
+					if (device_name) {
+						nutscan_add_option_to_device(nut_dev,
+							"product",
+							device_name);
+						free(device_name);
+						device_name = NULL;
+					}
+
+					if (serialnumber) {
+						nutscan_add_option_to_device(nut_dev,
+							"serial",
+							serialnumber);
+						free(serialnumber);
+						serialnumber = NULL;
+					}
+				} else {
+					sprintf(string, "%04X", VendorID);
+					nutscan_add_commented_option_to_device(nut_dev,
+						"vendorid",
+						string,
+						"");
+
+					sprintf(string, "%04X", ProductID);
+					nutscan_add_commented_option_to_device(nut_dev,
+						"productid",
+						string,
+						"");
+
+					if (device_name) {
+						nutscan_add_commented_option_to_device(nut_dev,
+							"product",
+							device_name,
+							"");
+						free(device_name);
+						device_name = NULL;
+					}
+
+					if (serialnumber) {
+						nutscan_add_commented_option_to_device(nut_dev,
+							"serial",
+							serialnumber,
+							"");
+						free(serialnumber);
+						serialnumber = NULL;
+					}
 				}
 
 				if (vendor_name) {
-					nutscan_add_option_to_device(nut_dev,
-						"vendor",
-						vendor_name);
+					/* NOTE: nutdrv_atcl_usb recognizes
+					 * "vendor" but not other opts */
+					if (strncmp(driver_name, "bcmxcp_usb", 10)
+					 && strncmp(driver_name, "richcomm_usb", 12)
+					) {
+						nutscan_add_option_to_device(nut_dev,
+							"vendor",
+							vendor_name);
+					} else {
+						nutscan_add_commented_option_to_device(nut_dev,
+							"vendor",
+							vendor_name,
+							"");
+					}
 					free(vendor_name);
 					vendor_name = NULL;
 				}
@@ -661,4 +887,9 @@ nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 	return NULL;
 }
 
+
+int nutscan_unload_usb_library(void)
+{
+	return 0;
+}
 #endif /* WITH_USB */
